@@ -13,7 +13,7 @@ COPYING distributed with the source files.
 #include <gtk/gtktable.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkentry.h>
-#include <gtk/gtkalignment.h>
+#include <gtk/gtkmisc.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkbbox.h>
 #include <gtk/gtkhbbox.h>
@@ -34,8 +34,8 @@ COPYING distributed with the source files.
 
 #include "addressbook.h"
 #include "dialogs.h"
-#include "utf8_utils.h"
-#include "tree_path_handle.h"
+#include "utils/utf8_utils.h"
+#include "utils/tree_path_handle.h"
 #include "addressbook_icons.h"
 
 #ifdef ENABLE_NLS
@@ -140,13 +140,13 @@ AddressBook::AddressBook(const int size, GtkWindow* parent_p):
   GtkTreeViewColumn* column_p =
     gtk_tree_view_column_new_with_attributes(gettext("Name"), renderer_p,
 					     "text", ModelColumns::name,
-					     0);
+					     static_cast<void*>(0));
   gtk_tree_view_append_column(tree_view_p, column_p);
   
   renderer_p = gtk_cell_renderer_text_new();
   column_p = gtk_tree_view_column_new_with_attributes(gettext("Number"), renderer_p,
 						      "text", ModelColumns::number,
-						      0);
+						      static_cast<void*>(0));
   gtk_tree_view_append_column(tree_view_p, column_p);
 
   GtkTreeSelection* selection_p = gtk_tree_view_get_selection(tree_view_p);
@@ -583,19 +583,14 @@ AddressDialog::AddressDialog(const int standard_size, GtkWindow* parent_p):
   ok_button_p = gtk_button_new_from_stock(GTK_STOCK_OK);
   cancel_button_p = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
 
-  GtkWidget* name_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* number_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-
   GtkWidget* name_label_p = gtk_label_new(gettext("Name:"));
   GtkWidget* number_label_p = gtk_label_new(gettext("Number:"));
 
   gtk_label_set_justify(GTK_LABEL(name_label_p), GTK_JUSTIFY_RIGHT);
   gtk_label_set_justify(GTK_LABEL(number_label_p), GTK_JUSTIFY_RIGHT);
 
-  gtk_container_add(GTK_CONTAINER(name_label_alignment_p),
-		    name_label_p);
-  gtk_container_add(GTK_CONTAINER(number_label_alignment_p),
-		    number_label_p);
+  gtk_misc_set_alignment(GTK_MISC(name_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(number_label_p), 1, 0.5);
 
   name_entry_p = gtk_entry_new();
   number_entry_p = gtk_entry_new();
@@ -611,7 +606,7 @@ AddressDialog::AddressDialog(const int standard_size, GtkWindow* parent_p):
   gtk_container_add(GTK_CONTAINER(button_box_p), cancel_button_p);
   gtk_container_add(GTK_CONTAINER(button_box_p), ok_button_p);
 
-  gtk_table_attach(table_p, name_label_alignment_p,
+  gtk_table_attach(table_p, name_label_p,
 		   0, 1, 0, 1,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/2, standard_size/4);
@@ -620,7 +615,7 @@ AddressDialog::AddressDialog(const int standard_size, GtkWindow* parent_p):
 		   GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_SHRINK,
 		   standard_size/2, standard_size/4);
 
-  gtk_table_attach(table_p, number_label_alignment_p,
+  gtk_table_attach(table_p, number_label_p,
 		   0, 1, 1, 2,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/2, standard_size/4);
