@@ -43,17 +43,23 @@ COPYING distributed with the source files.
 #define EFAX_GTK_FILE_DIALOG_NEW gtk_file_selection_new(0)
 #else
 #include <gtk/gtkfilechooserdialog.h>
-#define EFAX_GTK_FILE_DIALOG_NEW gtk_file_chooser_dialog_new(0, 0, GTK_FILE_CHOOSER_ACTION_OPEN, 0)
+#include <glib/glist.h>
+#define EFAX_GTK_FILE_DIALOG_NEW (gtk_file_chooser_dialog_new(0, 0, \
+                                 GTK_FILE_CHOOSER_ACTION_OPEN, \
+                                 gettext("View"), GTK_RESPONSE_NONE, \
+                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, \
+                                 GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, \
+                                 static_cast<void*>(0)))
 #endif
 
 #include <pango/pango-font.h>
 #include <pango/pango-layout.h>
 
 #include "dialogs.h"
-#include "shared_handle.h"
-#include "gobj_handle.h"
-#include "mutex.h"
-#include "utf8_utils.h"
+#include "utils/shared_handle.h"
+#include "utils/gobj_handle.h"
+#include "utils/mutex.h"
+#include "utils/utf8_utils.h"
 #include "gpl.h"
 
 #ifdef ENABLE_NLS
@@ -88,11 +94,8 @@ FileReadSelectDialog::FileReadSelectDialog(int size, bool multi_files, GtkWindow
 					GTK_WINDOW(EFAX_GTK_FILE_DIALOG_NEW)),
 				standard_size(size) {
 
-  GtkWidget* view_button_p = gtk_button_new_with_label(gettext("View"));
-  GTK_WIDGET_SET_FLAGS(view_button_p, GTK_CAN_DEFAULT);
-  g_signal_connect(G_OBJECT(view_button_p), "clicked",
-		   G_CALLBACK(FileReadSelectDialogCB::file_view_file), this);
-  
+  GtkWidget* view_button_p;
+
 #ifdef EFAX_GTK_USE_FILE_SEL
   GtkFileSelection* file_object_p = GTK_FILE_SELECTION(get_win());
 
@@ -109,9 +112,12 @@ FileReadSelectDialog::FileReadSelectDialog(int size, bool multi_files, GtkWindow
   gtk_box_set_homogeneous(hbox_p, false);
 
   GtkWidget* label_p = gtk_label_new(0);
+  view_button_p = gtk_button_new_with_label(gettext("View"));
   gtk_box_pack_start(hbox_p, label_p, true, false, 0);
   gtk_box_pack_start(hbox_p, view_button_p, false, false, 0);
 
+  GTK_WIDGET_SET_FLAGS(view_button_p, GTK_CAN_DEFAULT);
+
   ok_button_p = file_object_p->ok_button;
   cancel_button_p = file_object_p->cancel_button;
 
@@ -128,15 +134,19 @@ FileReadSelectDialog::FileReadSelectDialog(int size, bool multi_files, GtkWindow
   GtkBox* hbox_p = GTK_BOX(GTK_DIALOG(get_win())->action_area);
   gtk_box_set_homogeneous(hbox_p, true);
 
-  ok_button_p = gtk_button_new_from_stock(GTK_STOCK_OK);
-  cancel_button_p = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+  GList* button_list = gtk_container_get_children(GTK_CONTAINER(hbox_p));
 
-  GTK_WIDGET_SET_FLAGS(ok_button_p, GTK_CAN_DEFAULT);
-  GTK_WIDGET_SET_FLAGS(cancel_button_p, GTK_CAN_DEFAULT);
+  if (g_list_length(button_list) < 3) {
+    write_error("Button creation error in FileReadSelectDialog::FileReadSelectDialog()\n");
+    gtk_widget_show_all(GTK_WIDGET(get_win()));
+    g_list_free(button_list);
+    return;
+  }
+  ok_button_p = static_cast<GtkWidget*>(g_list_nth_data(button_list, 0));
+  cancel_button_p =  static_cast<GtkWidget*>(g_list_nth_data(button_list, 1));;
+  view_button_p =  static_cast<GtkWidget*>(g_list_nth_data(button_list, 2));;
 
-  gtk_box_pack_start(hbox_p, view_button_p, false, false, 0);
-  gtk_box_pack_start(hbox_p, cancel_button_p, false, false, 0);
-  gtk_box_pack_start(hbox_p, ok_button_p, false, false, 0);
+  g_list_free(button_list);
 
 #endif
 
@@ -144,6 +154,8 @@ FileReadSelectDialog::FileReadSelectDialog(int size, bool multi_files, GtkWindow
 		   G_CALLBACK(FileReadSelectDialogCB::file_selected), this);
   g_signal_connect(G_OBJECT(cancel_button_p), "clicked",
 		   G_CALLBACK(FileReadSelectDialogCB::file_selected), this);
+  g_signal_connect(G_OBJECT(view_button_p), "clicked",
+		   G_CALLBACK(FileReadSelectDialogCB::file_view_file), this);
 
   gtk_window_set_type_hint(get_win(), GDK_WINDOW_TYPE_HINT_DIALOG);
   gtk_window_set_position(get_win(), GTK_WIN_POS_CENTER_ON_PARENT);
