@@ -25,7 +25,7 @@ COPYING distributed with the source files.
 
 #include <sigc++/sigc++.h>
 
-#include "window.h"
+#include "utils/window.h"
 
 
 namespace { // we put the functions in anonymous namespace in dialogs.cpp
