@@ -30,10 +30,10 @@ COPYING distributed with the source files.
 #include <glib/gtimer.h>
 
 #include "efax_controller.h"
-#include "thread.h"
-#include "mutex.h"
-#include "utf8_utils.h"
-#include "shared_handle.h"
+#include "fax_list_manager.h"
+#include "utils/thread.h"
+#include "utils/mutex.h"
+#include "utils/shared_handle.h"
 
 #ifdef HAVE_STRINGSTREAM
 #include <sstream>
@@ -51,7 +51,8 @@ extern "C" pid_t getpgid(pid_t);
 
 
 EfaxController::EfaxController(void): state(inactive),
-				      close_down(false) {
+				      close_down(false),
+                                      received_fax_count(0) {
   // set up state_messages
 
   state_messages.push_back(gettext("Inactive"));
@@ -62,8 +63,6 @@ EfaxController::EfaxController(void): state(inactive),
   state_messages.push_back(gettext("Sending fax"));
   state_messages.push_back(gettext("Sending fax"));
 
-  cleanup_on_start();
-
   fax_made_notify.connect(sigc::mem_fun(*this, &EfaxController::sendfax_slot));
   no_fax_made_notify.connect(sigc::mem_fun(*this, &EfaxController::no_fax_made_slot));
 }
@@ -89,24 +88,21 @@ void EfaxController::init_sendfax_parms(void) {
   std::time(&time_count);
   time_p = std::localtime(&time_count);
 
-  char date_string[23];
-  const char format[] = "%a %d-%b-%Y %H:%M";
-  std::strftime(date_string, 23, format, time_p);
+  char date_string[150];
+  // create the date and time in an internationally acceptable
+  // numeric (ASCII) only character format which efax can make sense of
+  const char format[] = "%Y-%m-%d %H:%M";
+  std::strftime(date_string, sizeof(date_string), format, time_p);
 
   std::string temp("-h");
   temp += date_string;
   temp += "    ";
-  try {
-    temp += Utf8::locale_from_utf8(gettext("From"));
-    temp += " ";
-    temp += Utf8::locale_from_utf8(prog_config.my_name);
-  }
-  catch (Utf8::ConversionError&) {
-    write_error("UTF-8 conversion error in EfaxController::init_sendfax_parms()\n");
-  }
+  temp += prog_config.my_name;
+
   temp += " (";
   temp += prog_config.my_number + ") --> ";
-  temp += last_fax_item_sent.number + "    p.%d/%d";
+  if (prog_config.addr_in_header) temp += last_fax_item_sent.number;
+  temp += "      %d/%d";
   sendfax_parms_vec.push_back(temp);
 
   if (last_fax_item_sent.number.empty()) {
@@ -123,7 +119,7 @@ void EfaxController::init_sendfax_parms(void) {
   }
 }
 
-void EfaxController::sendfax(const Fax_item& fax_item) {
+void EfaxController::sendfax_impl(const Fax_item& fax_item, bool in_timer_event) {
 
   if (state == receive_standby) {
     if (!is_receiving_fax()) {
@@ -134,11 +130,7 @@ void EfaxController::sendfax(const Fax_item& fax_item) {
     else write_error(gettext("Cannot send fax - a fax is being received\n"));
   }
 
-  // note that this is in an else block - we do not want this
-  // tested if state was set to start_send_on_standby above - we 
-  // want that test to occur on the call to this method in
-  // EfaxController::timer_event()
-  else if (state == inactive || state == start_send_on_standby) {
+  else if (state == inactive || (in_timer_event && state == start_send_on_standby)) {
     stdout_pipe.open(PipeFifo::non_block);
     // efax is sensitive to timing, so set pipe write to non-block also
     stdout_pipe.make_write_non_block();
@@ -357,12 +349,7 @@ void EfaxController::make_fax_thread(void) {
     std::string::size_type pos = filename_iter->find_last_of('/');
 
     if (pos == std::string::npos || pos + 2 > filename_iter->size()) {
-      try {
-	write_error(Utf8::locale_from_utf8(gettext("Not valid file name\n")).c_str());
-      }
-      catch (Utf8::ConversionError&) {
-	write_error("UTF-8 conversion error in EfaxController::make_fax_thread()\n");
-      }
+      write_error(gettext("Not valid file name\n"));
 
       // synchronise memory on multi-processor systems before we emit the
       // Notifier signal - we call fax_made_sem.wait() in
@@ -375,12 +362,7 @@ void EfaxController::make_fax_thread(void) {
     }
     
     else if (access(filename_iter->c_str(), F_OK)) {
-      try {
-	write_error(Utf8::locale_from_utf8(gettext("File does not exist\n")).c_str());
-      }
-      catch (Utf8::ConversionError&) {
-	write_error("UTF-8 conversion error in EfaxController::make_fax_thread()\n");
-      }
+      write_error(gettext("File does not exist\n"));
 
       // synchronise memory on multi-processor systems before we emit the
       // Notifier signal - we call fax_made_sem.wait() in
@@ -393,12 +375,7 @@ void EfaxController::make_fax_thread(void) {
     }
 
     else if (access(filename_iter->c_str(), R_OK)) {
-      try {
-	write_error(Utf8::locale_from_utf8(gettext("User does not have read permission on the file\n")).c_str());
-      }
-      catch (Utf8::ConversionError&) {
-	write_error("UTF-8 conversion error in EfaxController::make_fax_thread()\n");
-      }
+      write_error(gettext("User does not have read permission on the file\n"));
 
       // synchronise memory on multi-processor systems before we emit the
       // Notifier signal - we call fax_made_sem.wait() in
@@ -568,12 +545,7 @@ void EfaxController::make_fax_thread(void) {
 #endif  
     
     if (!valid_file) {
-      try {
-	write_error(Utf8::locale_from_utf8(gettext("Not valid postscript file\n")).c_str());
-      }
-      catch (Utf8::ConversionError&) {
-	write_error("UTF-8 conversion error in EfaxController::make_fax_thread()\n");
-      }
+      write_error(gettext("Not valid postscript file\n"));
 
       // synchronise memory on multi-processor systems before we emit the
       // Notifier signal - we call fax_made_sem.wait() in
@@ -600,10 +572,9 @@ void EfaxController::make_fax_thread(void) {
 std::pair<const char*, char* const*> EfaxController::get_receive_parms(int mode) {
 
   std::vector<std::string> efax_parms(prog_config.parms);
+  std::string temp;
 
-  std::string temp("-r");
-  temp += prog_config.receive_dirname;
-  efax_parms.push_back(temp);
+  efax_parms.push_back("-rcurrent");
 
   if (mode == receive_takeover) efax_parms.push_back("-w");
       
@@ -634,69 +605,33 @@ std::pair<const char*, char* const*> EfaxController::get_receive_parms(int mode)
 
 void EfaxController::receive(State mode) {
 
-  if (state != inactive) beep();
+  // check pre-conditions
+  if (state != inactive) {
+    beep();
+    return;
+  }
+  if (prog_config.working_dir.empty()) {
+    write_error(gettext("You don't have the $HOME environmental variable set\n"));
+    beep();
+    return;
+  }
 
+  // now proceed to put the program in receive mode
+  std::string fax_pathname(prog_config.working_dir);
+  fax_pathname += "/faxin/current";
+  // we can extract the C string here - we do not do anything below to make it invalid
+  const char* fax_pathname_str = fax_pathname.c_str();
+
+  if (mkdir(fax_pathname_str, S_IRUSR | S_IWUSR | S_IXUSR) == -1) {
+    write_error(gettext("SERIOUS SYSTEM ERROR: Can't create directory to save received faxes.\n"
+			"Faxes cannot be received!\n"));
+  }
   else {
-    // get a time value to create the directory name into which the fax is to be saved
-    // and to insert in prog_config.receive_dirname for parent and child
-    struct std::tm* time_p;
-    std::time_t time_count;
-    
-    std::time(&time_count);
-    time_p = std::localtime(&time_count);
-
-    std::string fax_pathname;
-    if (!prog_config.working_dir.empty()) {
-      fax_pathname = prog_config.working_dir;
-      fax_pathname += "/faxin/";
-    }
-    else {
-      write_error("You don't have the $HOME environmental variable set.\n"
-		  "You may need to search for the directory in which\n"
-		  "the received fax is saved, and it probably won't\n"
-		  "appear in the fax list!\n");
-    }
-
-    const char format[] = "%y%m%d%H%M%S";
-    {
-      // lock the Prog_config object while we modify it
-      Thread::Mutex::Lock lock(*prog_config.mutex_p);
-      std::strftime(prog_config.receive_dirname, MAX_TEMP_NAME + 1, format, time_p);
-    }
-    std::string temp(fax_pathname);
-    temp += prog_config.receive_dirname;
-
-    // check whether directory already exists or can't be created
-    int count;
-    for (count = 0; count < 4 && mkdir(temp.c_str(), S_IRUSR | S_IWUSR | S_IXUSR); count++) {
-      // it is not worth checking for a short sleep because of a signal as we will
-      // repeat the attempt four times anyway
-      sleep(1); // wait a second to get a different time
-      std::time(&time_count);
-      time_p = std::localtime(&time_count);
-      {
-	// lock the Prog_config object while we modify it
-	Thread::Mutex::Lock lock(*prog_config.mutex_p);
-	std::strftime(prog_config.receive_dirname, MAX_TEMP_NAME + 1, format, time_p);
-      }
-      temp = fax_pathname + prog_config.receive_dirname;
-    }
-    if (count == 4) {
-      write_error("Can't create directory to save fax\n");
-      {
-	// lock the Prog_config object while we modify it
-	Thread::Mutex::Lock lock(*prog_config.mutex_p);
-	*prog_config.receive_dirname = 0;
-      }
-      return;
-    }
-
-    fax_pathname += prog_config.receive_dirname;
 
     stdout_pipe.open(PipeFifo::non_block);
     // efax is sensitive to timing, so set pipe write to non-block also
     stdout_pipe.make_write_non_block();
-    
+  
     state = mode;
     display_state();
 
@@ -722,7 +657,7 @@ void EfaxController::receive(State mode) {
       stdout_pipe.connect_to_stdout();
       connect_to_stderr();
 
-      chdir(fax_pathname.c_str());
+      chdir(fax_pathname_str);
 
       // wait before we call execvp() until the parent process has set itself up
       // and releases this process
@@ -773,6 +708,17 @@ void EfaxController::stop(void) {
 
 void EfaxController::timer_event(void) {
 
+  // if a fax list is in the process of being constructed, we need to wait until
+  // construction has completed before reaping efax exit status and acting on it
+  // (we might have reached here through the call to gtk_main_iteration() in
+  // FaxListManager::populate_fax_list())
+  if (FaxListManager::is_fax_received_list_main_iteration()
+      || FaxListManager::is_fax_sent_list_main_iteration()) {
+    return;
+  }
+
+  // preconditions are OK - proceed
+
   int stat_val;
   pid_t result;
 
@@ -797,7 +743,10 @@ void EfaxController::timer_event(void) {
       case EfaxController::sending:
       case EfaxController::send_on_standby:
 	if (!exit_code) {
-	  save_sent_fax();
+	  std::pair<std::string, std::string> fax_info(save_sent_fax());
+	  // notify that the fax has been sent
+	  if (!fax_info.first.empty()) fax_sent_notify(fax_info);
+
 	  if (last_fax_item_sent.is_socket_file) {
 	    // if we are sending a print job via the socket server, notify the
 	    // SocketServer object which created the temporary file (the server
@@ -821,9 +770,9 @@ void EfaxController::timer_event(void) {
 	}
 	break;
       case EfaxController::start_send_on_standby:
-	clear_receive_dirname();
+	receive_cleanup();
 	unjoin_child();
-	if (!close_down) sendfax(last_fax_item_sent);
+	if (!close_down) sendfax_impl(last_fax_item_sent, true);
 	else {
 	  state = inactive;
 	  display_state();
@@ -842,13 +791,17 @@ void EfaxController::timer_event(void) {
       }
 
       if (end_receive || restart_standby) {
-	clear_receive_dirname();
+	receive_cleanup();
 	unjoin_child();
 	state = inactive; // this is needed even if we are going to call receive()
 	// now restart if in standby mode
 	if (restart_standby) receive(receive_standby);
 	// we do not need to call display_state() if we have called receive(), as
-	// receive() will call display_state() itself
+	// receive() will call display_state() itself (calling display_state() will
+	// also update the received fax count displayed in the tray icon tooltip if we are
+	// in receive_standby mode - this will appear in the tray_item tooltip, because
+	// TrayIcon::set_tooltip_slot() is connected to the write_state signal in the
+	// MainWindow::MainWindow() constructor)
 	else display_state();
 	restart_standby = false;
 	end_receive = false;
@@ -866,46 +819,161 @@ void EfaxController::timer_event(void) {
   }
 }
 
-void EfaxController::clear_receive_dirname(void) {
-  if (*prog_config.receive_dirname) {
-    // now delete receive_dirname if it is empty
-    std::string full_dirname;
-    if (!prog_config.working_dir.empty()) {
-      full_dirname = prog_config.working_dir;
-      full_dirname += "/faxin/";
-    }
-    full_dirname += prog_config.receive_dirname;
-    int result = rmdir(full_dirname.c_str()); // only deletes the directory if it is empty
+void EfaxController::receive_cleanup(void) {
+  // delete the "current" receive directory if it is empty
+  std::string full_dirname(prog_config.working_dir);
+  full_dirname += "/faxin/current";
+  int result = rmdir(full_dirname.c_str()); // only deletes the directory if it is empty
 
-    if (result == -1 && errno == ENOTEMPTY) {
-      // before assuming a fax has been successfully received, check to see
-      // if [fax_name].001 is of 0 size (if it is delete it)
-      struct stat statinfo;
-      std::string file_name(full_dirname);
-      file_name += '/';
-      file_name += prog_config.receive_dirname;
-      file_name += ".001";
-      if (!stat(file_name.c_str(), &statinfo)
-	  && !statinfo.st_size) {
-	unlink(file_name.c_str());
-	rmdir(full_dirname.c_str());
+  if (result == -1 && errno == ENOTEMPTY) {
+    // before assuming a fax has been successfully received, check to see
+    // if current.001 is of 0 size (if it is delete it)
+    struct stat statinfo;
+    std::string file_name(full_dirname);
+    file_name += "/current.001";
+    if (!stat(file_name.c_str(), &statinfo)
+	&& !statinfo.st_size) {
+      unlink(file_name.c_str());
+      rmdir(full_dirname.c_str());
+    }
+    else { // we have received a fax - save it
+      std::pair<std::string, std::string> fax_info(save_received_fax());
+      // notify that the fax has been received
+      if (!fax_info.first.empty()) {
+	received_fax_count++;
+	fax_received_notify(fax_info);
       }
-      else fax_received_notify();
     }
-    // lock the Prog_config object while we modify it
-    Thread::Mutex::Lock lock(*prog_config.mutex_p);
-    *prog_config.receive_dirname = 0;
   }
 }
 
+std::pair<std::string, std::string> EfaxController::save_received_fax(void) {
+
+  char faxname[18];
+  *faxname = 0;
+
+  // get a time value to create the directory name into which the fax is to be saved
+  struct std::tm* time_p;
+  std::time_t time_count;
+    
+  std::time(&time_count);
+  time_p = std::localtime(&time_count);
+
+  std::string new_dirname(prog_config.working_dir);
+  new_dirname += "/faxin/";
+
+  const char format[] = "%Y%m%d%H%M%S";
+  std::strftime(faxname, sizeof(faxname), format, time_p);
+  std::string temp(new_dirname);
+  temp += faxname;
+
+  // check whether directory already exists or can't be created
+  int count;
+  for (count = 0; count < 4 && mkdir(temp.c_str(), S_IRUSR | S_IWUSR | S_IXUSR); count++) {
+    // it is not worth checking for a short sleep because of a signal as we will
+    // repeat the attempt four times anyway
+    sleep(1); // wait a second to get a different time
+    std::time(&time_count);
+    time_p = std::localtime(&time_count);
+    std::strftime(faxname, sizeof(faxname), format, time_p);
+    temp = new_dirname + faxname;
+  }
+  if (count == 4) {
+    write_error(gettext("SERIOUS SYSTEM ERROR: Can't create directory to save received fax.\n"
+			"This fax will not be saved!\n"));
+    *faxname = 0;
+  }
+
+  else { // move the faxes into the new directory
+
+    // make new_dirname the same as the directory name for the saved faxes
+    // that we have just created
+    new_dirname += faxname;
+
+    std::string old_dirname(prog_config.working_dir);
+    old_dirname += "/faxin/current";
+
+    std::vector<std::string> filelist;
+    struct dirent* direntry;
+    struct stat statinfo;
+
+    DIR* dir_p;
+    if ((dir_p = opendir(old_dirname.c_str())) == 0) {
+      write_error(gettext("SERIOUS SYSTEM ERROR: Can't open received fax directory.\n"
+			  "This fax will not be saved!\n"));
+    }
+
+    else {
+      chdir(old_dirname.c_str());
+      while ((direntry = readdir(dir_p)) != 0) {
+	stat(direntry->d_name, &statinfo);
+	if (S_ISREG(statinfo.st_mode)) {
+	  filelist.push_back(direntry->d_name);
+	}
+      }
+
+      closedir(dir_p);
+
+      bool failure = false;
+      std::vector<std::string>::const_iterator iter;
+      std::string old_filename;
+      std::string new_filename;
+
+      for (iter = filelist.begin(); iter != filelist.end(); ++iter) {
+	old_filename = old_dirname;
+	old_filename += '/';
+	old_filename += *iter;
+	if (!iter->substr(0, std::strlen("current.")).compare("current.")) {
+	  new_filename = new_dirname;
+	  new_filename += '/';
+	  new_filename += faxname;
+	  new_filename += iter->substr(std::strlen("current"));  // add the suffix (".001" etc.)
+	  if (link(old_filename.c_str(), new_filename.c_str())) {
+	    failure = true;
+	    break;
+	  }
+	}
+      }
+      if (failure) {
+	write_error(gettext("SERIOUS SYSTEM ERROR: Cannot save all of the received fax.\n"
+			    "All or part of the fax will be missing!\n"));
+      }
+      else {
+	for (iter = filelist.begin(); iter != filelist.end(); ++iter) {
+	  old_filename = old_dirname;
+	  old_filename += '/';
+	  old_filename += *iter;
+	  unlink(old_filename.c_str());
+	}
+	if (rmdir(old_dirname.c_str())) {
+	  std::string msg("Can't delete directory ");
+	  msg += old_dirname;
+	  msg += "\nThe contents should have been moved to ";
+	  msg += new_dirname;
+	  msg += "\nand it should now be empty -- please check\n";
+	  write_error(msg.c_str());
+	}
+      }	
+    }
+
+    // reset current directory
+    std::string temp(prog_config.working_dir + "/faxin");
+    chdir(temp.c_str());
+  }
+  // this program does not provide an automatic fax description at the moment
+  // so pass an empty string as the secong member of the pair
+  return std::pair<std::string, std::string>(std::string(faxname), std::string());
+}
+
 bool EfaxController::read_pipe_slot(void) {
 
   char pipe_buffer[PIPE_BUF + 1];
   ssize_t result;
 
   while ((result = stdout_pipe.read(pipe_buffer, PIPE_BUF)) > 0) {
-    pipe_buffer[result] = 0;
-    stdout_message(pipe_buffer);
+    SharedHandle<char*> output_h(stdout_pipe_reassembler(pipe_buffer, result));
+    if (output_h.get()) stdout_message(output_h.get());
+    else write_error(gettext("Invalid Utf8 received in EfaxController::read_pipe_slot()\n"));
   }
   return true; // this is multi-shot
 }
@@ -926,6 +994,7 @@ void EfaxController::unjoin_child(void) {
 
   g_source_remove(iowatch_tag);
   stdout_pipe.close();
+  stdout_pipe_reassembler.reset();
 }
 
 void EfaxController::kill_child(void) {
@@ -954,8 +1023,12 @@ void EfaxController::kill_child(void) {
   }
 }
 
-void EfaxController::save_sent_fax(void) {
-  
+std::pair<std::string, std::string> EfaxController::save_sent_fax(void) {
+
+  char faxname[18];
+  *faxname = 0;
+  std::string description;
+
   // get a time value to create the directory name into which the fax is to be saved
   struct std::tm* time_p;
   std::time_t time_count;
@@ -964,23 +1037,13 @@ void EfaxController::save_sent_fax(void) {
   time_p = std::localtime(&time_count);
 
   // now create the directory into which the fax files to be saved
-  std::string fileout_name;
-  if (!prog_config.working_dir.empty()) {
-    fileout_name = prog_config.working_dir;
-    fileout_name += "/faxsent/";
-  }
-  else {
-    write_error("You don't have the $HOME environmental variable set.\n"
-		"You may need to search for the directory in which\n"
-		"the fax is saved, and it probably won't appear in\n"
-		"the fax list!\n");
-  }
+  std::string fileout_name(prog_config.working_dir);
+  fileout_name += "/faxsent/";
 
-  const char dirname_format[] = "%y%m%d%H%M%S";
-  char top_dirname[MAX_TEMP_NAME + 1];
-  std::strftime(top_dirname, MAX_TEMP_NAME + 1, dirname_format, time_p);
+  const char dirname_format[] = "%Y%m%d%H%M%S";
+  std::strftime(faxname, sizeof(faxname), dirname_format, time_p);
   std::string temp(fileout_name);
-  temp += top_dirname;
+  temp += faxname;
 
   // check whether directory already exists or can't be created
   int count;
@@ -990,165 +1053,182 @@ void EfaxController::save_sent_fax(void) {
     sleep(1); // wait a second to get a different time
     std::time(&time_count);
     time_p = std::localtime(&time_count);
-    std::strftime(top_dirname, MAX_TEMP_NAME + 1, dirname_format, time_p);
-    temp = fileout_name + top_dirname;
+    std::strftime(faxname, sizeof(faxname), dirname_format, time_p);
+    temp = fileout_name + faxname;
   }
   if (count == 4) {
-    write_error("Can't create directory to save fax\n");
-    return;
+    write_error(gettext("SERIOUS SYSTEM ERROR: Can't create directory to save sent fax.\n"
+			"This fax will not be saved!\n"));
+    *faxname = 0;
   }
 
-  // now make fileout_name the same as the directory name for the saved faxes
-  // that we have just created
-  fileout_name += top_dirname;
-  const std::string fileout_dirname(fileout_name); // keep for later to enter a description
+  else {
+    // now make fileout_name the same as the directory name for the saved faxes
+    // that we have just created
+    fileout_name += faxname;
+    const std::string fileout_dirname(fileout_name); // keep for later to enter a description
 
-  // and now complete the unsuffixed file name of the destination files
-  fileout_name += '/';
-  fileout_name += top_dirname;
+    // and now complete the unsuffixed file name of the destination files
+    fileout_name += '/';
+    fileout_name += faxname;
 
-  // now create a string containing the date for the fax description
-  // glibc has const struct tm* as last param of strftime()
-  // but the const does not appear to be guaranteed by POSIX
-  // so do localtime() again just in case
-  time_p = std::localtime(&time_count);
-  const char date_description_format[] = "(%H%M %Z %d %b %Y)";
-  const int max_description_datesize = 126;
-  char date_description[max_description_datesize];
-  std::strftime(date_description, max_description_datesize, date_description_format, time_p);
+    // now create a string containing the date for the fax description
+    // glibc has const struct tm* as last param of strftime()
+    // but the const does not appear to be guaranteed by POSIX
+    // so do localtime() again just in case
+    time_p = std::localtime(&time_count);
+    const char date_description_format[] = "(%H%M %Z %d %b %Y)";
+    const int max_description_datesize = 126;
+    char date_description[max_description_datesize];
+    std::strftime(date_description, max_description_datesize, date_description_format, time_p);
 
-  // now copy files into the relevant directory for the fax pages to be saved
+    // now copy files into the relevant directory for the fax pages to be saved
 
-  std::vector<std::string> file_basename_vec; // this is used in making the fax description
-  int out_partnumber = 1; // in order to generate the suffix for each fax page as saved
+    std::vector<std::string> file_basename_vec; // this is used in making the fax description
+    int out_partnumber = 1; // in order to generate the suffix for each fax page as saved
+    
+    std::vector<std::string>::const_iterator sentname_iter;
+    for (sentname_iter = last_fax_item_sent.file_list.begin();
+	 sentname_iter != last_fax_item_sent.file_list.end(); ++sentname_iter) {
 
-  std::vector<std::string>::const_iterator sentname_iter;
-  for (sentname_iter = last_fax_item_sent.file_list.begin();
-       sentname_iter != last_fax_item_sent.file_list.end(); ++sentname_iter) {
+      std::string::size_type pos = sentname_iter->find_last_of('/');
+      if (pos == std::string::npos || pos + 2 > sentname_iter->size()) {
+	write_error("Not valid file name to save -- can't save sent fax\n");
+      }
 
-    std::string::size_type pos = sentname_iter->find_last_of('/');
-    if (pos == std::string::npos || pos + 2 > sentname_iter->size()) {
-      write_error("Not valid file name to save -- can't save sent fax\n");
-    }
+      else {
+	// save the file base name for later use in making the fax description
+	pos++;
+	file_basename_vec.push_back(sentname_iter->substr(pos));
 
-    else {
-      // save the file base name for later use in making the fax description
-      pos++;
-      file_basename_vec.push_back(sentname_iter->substr(pos));
-
-      // make the suffixes
-      int in_partnumber = 1;
+	// make the suffixes
+	int in_partnumber = 1;
 #ifdef HAVE_STRINGSTREAM
-      std::ostringstream in_suffix_strm;
-      in_suffix_strm << '.' << std::setfill('0')
-		     << std::setw(3) << in_partnumber;
-
-      std::ostringstream out_suffix_strm;
-      out_suffix_strm << '.' << std::setfill('0')
-		      << std::setw(3) << out_partnumber;
-
-      // make the suffixed source and destination files
-      std::string suffixed_inname = *sentname_iter + in_suffix_strm.str();
-      std::string suffixed_outname = fileout_name + out_suffix_strm.str();
-
-#else
-      std::ostrstream in_suffix_strm;
-      in_suffix_strm << '.' << std::setfill('0')
-		     << std::setw(3) << in_partnumber << std::ends;
-      const char* in_suffix = in_suffix_strm.str();
-
-      std::ostrstream out_suffix_strm;
-      out_suffix_strm << '.' << std::setfill('0')
-		      << std::setw(3) << out_partnumber << std::ends;
-      const char* out_suffix = out_suffix_strm.str();
-
-      // make the suffixed source and destination files
-      std::string suffixed_inname = *sentname_iter + in_suffix;
-      std::string suffixed_outname = fileout_name + out_suffix;
-
-      delete[] in_suffix;
-      delete[] out_suffix;
-#endif
-
-      std::ifstream filein;
-      std::ofstream fileout;
-      const int BLOCKSIZE = 1024;
-      char block[BLOCKSIZE];
-
-      while (!access(suffixed_inname.c_str(), R_OK)
-	     && (filein.open(suffixed_inname.c_str(), std::ios::in), filein) // use comma operator to check filein
-	     && (fileout.open(suffixed_outname.c_str(), std::ios::out), fileout)) { // ditto for fileout
-	while (filein) {
-	  filein.read(block, BLOCKSIZE);
-	  fileout.write(block, filein.gcount());
-	}
-	filein.close();
-	filein.clear();
-	fileout.close();
-	fileout.clear();
-	unlink(suffixed_inname.c_str());
-     
-	in_partnumber++;
-	out_partnumber++;
-#ifdef HAVE_STRINGSTREAM
-	in_suffix_strm.str("");
+	std::ostringstream in_suffix_strm;
 	in_suffix_strm << '.' << std::setfill('0')
 		       << std::setw(3) << in_partnumber;
 
-	out_suffix_strm.str("");
+	std::ostringstream out_suffix_strm;
 	out_suffix_strm << '.' << std::setfill('0')
 			<< std::setw(3) << out_partnumber;
 
 	// make the suffixed source and destination files
-	suffixed_inname = *sentname_iter + in_suffix_strm.str();
-	suffixed_outname = fileout_name + out_suffix_strm.str();
+	std::string suffixed_inname = *sentname_iter + in_suffix_strm.str();
+	std::string suffixed_outname = fileout_name + out_suffix_strm.str();
 
 #else
 	std::ostrstream in_suffix_strm;
 	in_suffix_strm << '.' << std::setfill('0')
 		       << std::setw(3) << in_partnumber << std::ends;
-	in_suffix = in_suffix_strm.str();
+	const char* in_suffix = in_suffix_strm.str();
 
 	std::ostrstream out_suffix_strm;
 	out_suffix_strm << '.' << std::setfill('0')
 			<< std::setw(3) << out_partnumber << std::ends;
-	out_suffix = out_suffix_strm.str();
+	const char* out_suffix = out_suffix_strm.str();
 
 	// make the suffixed source and destination files
-	suffixed_inname = *sentname_iter + in_suffix;
-	suffixed_outname = fileout_name + out_suffix;
-
+	std::string suffixed_inname = *sentname_iter + in_suffix;
+	std::string suffixed_outname = fileout_name + out_suffix;
+	
 	delete[] in_suffix;
 	delete[] out_suffix;
 #endif
-      }
-      fileout.clear();
-      if (in_partnumber < 2) write_error("There was a problem saving all or part of the sent fax\n");
-    }
-  }
 
-  // now save the sent fax description
-  if (out_partnumber > 1) {
-    const std::string description_filename(fileout_dirname + "/Description");
-    std::ofstream fileout(description_filename.c_str(), std::ios::out);
-    if (fileout) {
-      if (last_fax_item_sent.is_socket_file) fileout << gettext("PRINT JOB");
-      else {
-	std::vector<std::string>::const_iterator filename_iter;
-	for (filename_iter = file_basename_vec.begin();
-	     filename_iter != file_basename_vec.end(); ++filename_iter) {
-      
-	  if (filename_iter != file_basename_vec.begin()) fileout << '+';
-	  fileout << *filename_iter;
+	std::ifstream filein;
+	std::ofstream fileout;
+	const int BLOCKSIZE = 1024;
+	char block[BLOCKSIZE];
+
+	while (!access(suffixed_inname.c_str(), R_OK)
+	       && (filein.open(suffixed_inname.c_str(), std::ios::in), filein) // use comma operator to check filein
+	       && (fileout.open(suffixed_outname.c_str(), std::ios::out), fileout)) { // ditto for fileout
+	  while (filein) {
+	    filein.read(block, BLOCKSIZE);
+	    fileout.write(block, filein.gcount());
+	  }
+	  filein.close();
+	  filein.clear();
+	  fileout.close();
+	  fileout.clear();
+	  unlink(suffixed_inname.c_str());
+     
+	  in_partnumber++;
+	  out_partnumber++;
+#ifdef HAVE_STRINGSTREAM
+	  in_suffix_strm.str("");
+	  in_suffix_strm << '.' << std::setfill('0')
+			 << std::setw(3) << in_partnumber;
+
+	  out_suffix_strm.str("");
+	  out_suffix_strm << '.' << std::setfill('0')
+			  << std::setw(3) << out_partnumber;
+
+	  // make the suffixed source and destination files
+	  suffixed_inname = *sentname_iter + in_suffix_strm.str();
+	  suffixed_outname = fileout_name + out_suffix_strm.str();
+
+#else
+	  std::ostrstream in_suffix_strm;
+	  in_suffix_strm << '.' << std::setfill('0')
+			 << std::setw(3) << in_partnumber << std::ends;
+	  in_suffix = in_suffix_strm.str();
+
+	  std::ostrstream out_suffix_strm;
+	  out_suffix_strm << '.' << std::setfill('0')
+			  << std::setw(3) << out_partnumber << std::ends;
+	  out_suffix = out_suffix_strm.str();
+
+	  // make the suffixed source and destination files
+	  suffixed_inname = *sentname_iter + in_suffix;
+	  suffixed_outname = fileout_name + out_suffix;
+
+	  delete[] in_suffix;
+	  delete[] out_suffix;
+#endif
 	}
+	fileout.clear();
+	if (in_partnumber < 2) write_error("There was a problem saving all or part of the sent fax\n");
       }
-      if (!last_fax_item_sent.number.empty()) {
-	fileout << " --> " << last_fax_item_sent.number;
-      }
-      fileout << ' ' << date_description;
     }
-    else write_error("Cannot save sent fax description\n");
+
+    // now save the sent fax description
+    if (out_partnumber > 1) {
+      const std::string description_filename(fileout_dirname + "/Description");
+      std::ofstream fileout(description_filename.c_str(), std::ios::out);
+      if (fileout) {
+	if (last_fax_item_sent.is_socket_file) description = gettext("PRINT JOB");
+	else {
+	  std::vector<std::string>::const_iterator filename_iter;
+	  for (filename_iter = file_basename_vec.begin();
+	       filename_iter != file_basename_vec.end(); ++filename_iter) {
+      
+	    if (filename_iter != file_basename_vec.begin()) description += '+';
+	    try {
+	      description += Utf8::filename_to_utf8(*filename_iter);
+	    }
+	    catch (Utf8::ConversionError&) {
+	      write_error("UTF-8 conversion error in EfaxController::save_sent_fax()\n");
+	    }
+	  }
+	}
+	if (!last_fax_item_sent.number.empty()) {
+	  description += " --> ";
+	  description += last_fax_item_sent.number;
+	}
+	description += ' ';
+	try {
+	  description += Utf8::locale_to_utf8(date_description);
+	}
+	catch (Utf8::ConversionError&) {
+	  write_error("UTF-8 conversion error in EfaxController::save_sent_fax()\n");
+	}
+	fileout << description;
+      }
+      else write_error("Cannot save sent fax description\n");
+    }
   }
+  return std::pair<std::string, std::string>(std::string(faxname), description);
 }
 
 void EfaxController::cleanup_fax_send_fail(void) {
@@ -1198,66 +1278,12 @@ void EfaxController::cleanup_fax_send_fail(void) {
 bool EfaxController::is_receiving_fax(void) const {
 
   bool return_val = false;
-  if (*prog_config.receive_dirname) {
 
-    std::string file_name(prog_config.working_dir);
-    file_name += "/faxin/";
-    file_name += prog_config.receive_dirname;
-    file_name += '/';
-    file_name += prog_config.receive_dirname;
-    file_name += ".001";
+  std::string file_name(prog_config.working_dir);
+  file_name += "/faxin/current/current.001";
+
+  struct stat statinfo;
+  if (!stat(file_name.c_str(), &statinfo)) return_val = true;
 
-    struct stat statinfo;
-    if (!stat(file_name.c_str(), &statinfo)) return_val = true;
-  }
   return return_val;
 }
-
-void EfaxController::cleanup_on_start(void) {
-
-  // this program cleans up for closure by ordinary user action, and
-  // atexit_cleanup() in main.cpp caters for cases where the X session
-  // has brought about the closure of this program.  It is also possible
-  // that no clean-up at all will occur if the program is terminated on,
-  // say, a power failure.  In that case we may be left with an empty
-  // fax directory in the faxin directory - if so, let's delete it
-
-#ifndef PATH_MAX
-#define PATH_MAX 1023
-#endif
-  int count;
-  void* success;
-  std::string old_working_dir;
-  for (count = 1, success = 0; !success && count < 10; count++) {
-    ScopedHandle<char*> path_buffer_h(new char[(PATH_MAX * count) + 1]);
-    success = getcwd(path_buffer_h.get(), PATH_MAX * count);
-    if (success) old_working_dir = path_buffer_h.get();
-  }
-
-  if (success) {
-    std::string new_dir(prog_config.working_dir);
-    new_dir += "/faxin";
-    chdir(new_dir.c_str());
-
-    DIR* dir_p;
-    if ((dir_p = opendir(new_dir.c_str()))) {
-
-      struct dirent* direntry;
-      struct stat statinfo;
-
-      while ((direntry = readdir(dir_p)) != 0) {
-	stat(direntry->d_name, &statinfo);
-	if (S_ISDIR(statinfo.st_mode)
-	    // keep the check for the "oldfax" sub-directory even though it is not used
-	    // in version 2.2.13 onwards (users may still have it lying around)
-	    && std::strcmp(direntry->d_name, "oldfax")
-	    && std::strcmp(direntry->d_name, ".")
-	    && std::strcmp(direntry->d_name, "..")) {
-	  // rmdir only deletes a directory if it is empty
-	  rmdir(direntry->d_name);
-	}
-      }
-    }
-    chdir(old_working_dir.c_str());
-  }
-}
