@@ -27,10 +27,13 @@ COPYING distributed with the source files.
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkbox.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtkhbox.h>
 #include <gtk/gtkbbox.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtkpaned.h>
 #include <gtk/gtkhpaned.h>
+#include <gtk/gtkframe.h>
+#include <gtk/gtkmisc.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtktoolbar.h>
 #include <gtk/gtktooltips.h>
@@ -38,6 +41,7 @@ COPYING distributed with the source files.
 #include <gtk/gtkimage.h>
 #include <gtk/gtkstock.h>
 
+#include <gdk/gdkcolor.h>
 #include <gdk/gdkpixbuf.h>
 #include <gdk/gdkkeysyms.h> // the key codes are here
 
@@ -46,11 +50,11 @@ COPYING distributed with the source files.
 #include "fax_list.h"
 #include "dialogs.h"
 #include "fax_list_icons.h"
-#include "pipes.h"
-#include "thread.h"
-#include "mutex.h"
-#include "shared_handle.h"
-#include "utf8_utils.h"
+#include "utils/pipes.h"
+#include "utils/thread.h"
+#include "utils/mutex.h"
+#include "utils/shared_handle.h"
+#include "utils/utf8_utils.h"
 
 #ifdef HAVE_STRINGSTREAM
 #include <sstream>
@@ -94,8 +98,9 @@ void FaxListDialogCB::fax_list_button_clicked(GtkWidget* widget_p, void* data) {
   else if (widget_p == instance_p->delete_folder_button_p) {
     instance_p->delete_folder_prompt();
   }
-  else if (widget_p == instance_p->refresh_button_p) {
-    instance_p->refresh();
+  else if (widget_p == instance_p->reset_button_p) {
+    instance_p->reset_sig();
+    instance_p->display_new_fax_count();
   }
   else {
     write_error("Callback error in FaxListDialogCB::fax_list_book_button_clicked()\n");
@@ -123,7 +128,6 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   empty_trash_button_p = gtk_button_new();
   add_folder_button_p = gtk_button_new();
   delete_folder_button_p = gtk_button_new();
-  refresh_button_p = gtk_button_new();
 
   GtkWidget* button_box_p = gtk_hbutton_box_new();
   GtkWidget* fax_list_box_p = gtk_vbox_new(false, 0);
@@ -208,6 +212,8 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   }
   gtk_toolbar_append_widget(toolbar_p, add_folder_button_p,
 			    gettext("Add new folder"), "");
+  gtk_widget_set_sensitive(add_folder_button_p, true);
+  gtk_button_set_relief(GTK_BUTTON(add_folder_button_p), GTK_RELIEF_NORMAL);
 
   { // scope block for the GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_new_from_xpm_data(delete_folder_xpm));
@@ -219,11 +225,6 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
   gtk_widget_set_sensitive(delete_folder_button_p, false);
   gtk_button_set_relief(GTK_BUTTON(delete_folder_button_p), GTK_RELIEF_NONE);
 
-  image_p = gtk_image_new_from_stock(GTK_STOCK_REFRESH, efax_gtk_button_size);
-  gtk_container_add(GTK_CONTAINER(refresh_button_p), image_p);
-  gtk_toolbar_append_widget(toolbar_p, refresh_button_p,
-			    gettext("Refresh fax list"), "");
-
   gtk_container_add(GTK_CONTAINER(button_box_p), close_button_p);
 
   gtk_paned_pack1(GTK_PANED(list_pane_p),
@@ -246,9 +247,6 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
 		   GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_SHRINK,
 		   standard_size/3, standard_size/3);
 
-  gtk_box_pack_start(GTK_BOX(fax_list_box_p), GTK_WIDGET(toolbar_p), false, false, 0);
-  gtk_box_pack_start(GTK_BOX(fax_list_box_p), GTK_WIDGET(table_p), true, true, 0);
-  
   g_signal_connect(G_OBJECT(close_button_p), "clicked",
 		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
   g_signal_connect(G_OBJECT(print_button_p), "clicked",
@@ -265,9 +263,53 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
 		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
   g_signal_connect(G_OBJECT(delete_folder_button_p), "clicked",
 		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
-  g_signal_connect(G_OBJECT(refresh_button_p), "clicked",
-		   G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
 
+  GtkWidget* toolbar_box_p = gtk_hbox_new(false, 0);
+  gtk_box_pack_start(GTK_BOX(toolbar_box_p), GTK_WIDGET(toolbar_p), true, true, 0);
+
+  // if mode == FaxListEnum::sent we don't do anything at the moment
+  if (mode == FaxListEnum::received) {
+
+    // first provide a reset button
+    reset_button_p = gtk_button_new();
+    image_p = gtk_image_new_from_stock(GTK_STOCK_UNDO, efax_gtk_button_size);
+    gtk_container_add(GTK_CONTAINER(reset_button_p), image_p);
+    gtk_toolbar_append_widget(toolbar_p, reset_button_p,
+			      gettext("Reset new faxes count"), "");
+    gtk_widget_set_sensitive(reset_button_p, true);
+    gtk_button_set_relief(GTK_BUTTON(reset_button_p), GTK_RELIEF_NORMAL);
+    g_signal_connect(G_OBJECT(reset_button_p), "clicked",
+		     G_CALLBACK(FaxListDialogCB::fax_list_button_clicked), this);
+
+    // this is to put a label at the far right of the toolbar to give the number
+    // of received faxes since the fax list was last closed
+    // it would be easier to do this using GtkSeparatorToolItem but that
+    // is only available with GTK+ 2.4 onwards and I want to keep compatibility
+    // with earlier versions, so make a new toolbar for the purpose and put the
+    // toolbars in a GtkHBox object
+    GtkWidget* new_fax_toolbar_p = gtk_toolbar_new();
+    gtk_box_pack_end(GTK_BOX(toolbar_box_p), new_fax_toolbar_p, false, false, 0);
+
+    // now provide a display for the new received fax count
+    GtkWidget* new_fax_frame_p = gtk_frame_new(0);
+    gtk_container_set_border_width(GTK_CONTAINER(new_fax_frame_p), 1);
+    gtk_toolbar_append_widget(GTK_TOOLBAR(new_fax_toolbar_p), new_fax_frame_p, "", "");
+    new_fax_label_p = gtk_label_new(0);
+    gtk_misc_set_padding(GTK_MISC(new_fax_label_p), 6, 2);
+    gtk_container_add(GTK_CONTAINER(new_fax_frame_p), new_fax_label_p);
+    gtk_label_set_justify(GTK_LABEL(new_fax_label_p), GTK_JUSTIFY_RIGHT);
+    // as this is in the constructor, get_new_fax_count() always shows a 0 count
+    // but this is OK as we are using it here to size the label - the user must
+    // call display_new_fax_count() again once get_new_fax_count_sig has been
+    // connected up
+    display_new_fax_count();
+  }
+  else reset_button_p = 0;
+
+  // now put the toolbar hbox and the main table into the window vbox
+  gtk_box_pack_start(GTK_BOX(fax_list_box_p), GTK_WIDGET(toolbar_box_p), false, false, 0);
+  gtk_box_pack_start(GTK_BOX(fax_list_box_p), GTK_WIDGET(table_p), true, true, 0);
+  
   // now connect up the signal which indicates a selection has been made
   fax_list_manager.selection_notify.connect(sigc::mem_fun(*this, &FaxListDialog::set_buttons_slot));
 
@@ -286,11 +328,6 @@ FaxListDialog::FaxListDialog(FaxListEnum::Mode mode_, const int standard_size_):
 
   gtk_window_set_default_size(get_win(), standard_size * 18, standard_size * 16);
   gtk_widget_show_all(GTK_WIDGET(get_win()));
-
-  gtk_button_set_relief(GTK_BUTTON(add_folder_button_p), GTK_RELIEF_NORMAL);
-  gtk_button_set_relief(GTK_BUTTON(refresh_button_p), GTK_RELIEF_NORMAL);
-  gtk_widget_set_sensitive(add_folder_button_p, true);
-  gtk_widget_set_sensitive(refresh_button_p, true);
 }
 
 FaxListDialog::~FaxListDialog(void) {
@@ -363,12 +400,7 @@ void FaxListDialog::delete_fax(void) {
       std::string msg;
 
       if (fax_count == 1) {
-	GcharSharedHandle fax_number_h(fax_list_manager.get_fax_number());
-	msg = gettext("Permanently delete fax ");
-	// as we checked for a fax selection, fax_number_h should not be
-	// null, but check it in case
-	if (fax_number_h.get()) msg += fax_number_h.get();
-	msg += gettext("?\n");
+	msg = gettext("Permanently delete selected fax?\n");
 	msg += gettext("\n(NOTE: This will permanently delete the fax\n"
 		       "from the file system)");
       }
@@ -486,15 +518,9 @@ void FaxListDialog::print_fax_prompt(void) {
     // after fork()ing so there isn't even concurrent access
     if (prog_config.print_popup) {
       
-      GcharSharedHandle fax_number_h(fax_list_manager.get_fax_number());
-      std::string msg(gettext("Print fax "));
-      // as we checked for a fax selection, fax_number_h should not be
-      // null, but check it in case
-      if (fax_number_h.get()) msg += fax_number_h.get();
-      msg += gettext("?");
-      
+      std::string msg(gettext("Print selected fax?"));
       PromptDialog* dialog_p = new PromptDialog(msg.c_str(), gettext("efax-gtk: Print fax"),
-						  standard_size, get_win());
+						standard_size, get_win());
       dialog_p->accepted.connect(sigc::mem_fun(*this, &FaxListDialog::print_fax));
       // there is no memory leak -- the memory will be deleted when PromptDialog closes
     }
@@ -892,7 +918,14 @@ void FaxListDialog::print_fax_thread(void) {
 
       // now create a blocking pipe which the print processes can use to
       // communicate via stdout/stdin
-      PipeFifo fork_pipe(PipeFifo::block);
+      PipeFifo fork_pipe;
+      try {
+	fork_pipe.open(PipeFifo::block);
+      }
+      catch (PipeError&) {
+	write_error("Cannot open pipe in FaxListDialog::print_fax_thread()\n");
+	_exit(PIPE_ERROR);
+      }
       
       pid_t pid = fork();
 
@@ -993,7 +1026,7 @@ void FaxListDialog::view_fax_thread(void) {
   if (file_fd == -1) {
     write_error("Failed to make temporary file:\n"
 		"please check permissions in /tmp directory");
-    _exit(0);
+    return;
   }
 
   // now create a pipe and proceed to fork to write the postscript to temporary file
@@ -1014,7 +1047,7 @@ void FaxListDialog::view_fax_thread(void) {
 
   if (pid == -1) {
     write_error("Fork error - exiting\n");
-    _exit(FORK_ERROR);
+    std::exit(FORK_ERROR);
   }
     
   if (!pid) { // child process which will send postscript to stdout
@@ -1126,6 +1159,42 @@ void FaxListDialog::view_fax_thread(void) {
   }
 }
 
+void FaxListDialog::display_new_fax_count(void) {
+
+  // if mode == FaxListEnum::sent we don't do anything at the moment
+  if (mode == FaxListEnum::received) {
+    int fax_count = get_new_fax_count_sig();
+    
+    // Note: if fax_count == 0 we could pass NULL as the colour argument to
+    // gtk_widget_modify_fg() to bring up the default colour, but unfortunately
+    // this does not work with GTK+ 2.0, so we will explicitly allocate a black
+    // colour where fax_count == 0
+    GdkColor fg_colour;
+    if (fax_count) fg_colour.red = static_cast<guint16>(0.9 * 65535.0); // red colour
+    else fg_colour.red = 0;                                             // black colour
+
+    fg_colour.green = 0;
+    fg_colour.blue = 0;
+    // we don't need to allocate the GdkColor fg_colour object before it is
+    // used by gtk_widget_modify_fg() - that function will do it itself
+    gtk_widget_modify_fg(new_fax_label_p, GTK_STATE_NORMAL, &fg_colour);
+
+#ifdef HAVE_STRINGSTREAM
+    std::ostringstream strm;
+    strm << gettext("New faxes:");
+    strm << std::setfill(' ') << std::setw(4) << fax_count;
+    gtk_label_set_text(GTK_LABEL(new_fax_label_p), strm.str().c_str());
+#else
+    std::ostrstream strm;
+    strm << gettext("New faxes: ");
+    strm << std::setfill(' ') << std::setw(4) << fax_count << std::ends;
+    const char* text = strm.str();
+    gtk_label_set_text(GTK_LABEL(new_fax_label_p), text);
+    delete[] text;
+#endif
+  }
+}
+
 void FaxListDialog::delete_parms(std::pair<const char*, char* const*> parms_pair) {
 
   delete[] parms_pair.first;
@@ -1136,12 +1205,6 @@ void FaxListDialog::delete_parms(std::pair<const char*, char* const*> parms_pair
   delete[] parms_pair.second;
 }
 
-void FaxListDialog::refresh(void) {
-
-  fax_list_manager.refresh();
-  set_buttons_slot();
-}
-
 namespace { // callbacks for internal use only
 
 void EntryDialogCB::entry_dialog_selected(GtkWidget* widget_p, void* data) {
