@@ -9,6 +9,7 @@ COPYING distributed with the source files.
 #include <vector>
 #include <list>
 #include <cstring>
+#include <ctime>
 
 #include <unistd.h>
 #include <sys/types.h>
@@ -26,7 +27,6 @@ COPYING distributed with the source files.
 #include <gtk/gtkversion.h>
 
 #include <gdk/gdkpixmap.h>
-#include <glib/gunicode.h>
 #include <glib/gmain.h>
 
 // a guess for the future!
@@ -41,8 +41,8 @@ COPYING distributed with the source files.
 
 #include "fax_list_manager.h"
 #include "fax_list_manager_icons.h"
-#include "tree_path_handle.h"
-#include "utf8_utils.h"
+#include "utils/tree_path_handle.h"
+#include "utils/utf8_utils.h"
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -54,12 +54,19 @@ COPYING distributed with the source files.
 #define WRITE_PATH_TIMER_INTERVAL 300000    // number of milliseconds between flushing of logfile
                                             // = 5 minutes
 
+// strptime() is a POSIX rather than standard C function so it may not
+// be declared in <ctime>: so declare it here (with C linkage)
+extern "C" char* strptime(const char*, const char*, struct std::tm*);
+
+bool FaxListManager::fax_received_list_main_iteration = false;
+bool FaxListManager::fax_sent_list_main_iteration = false;
+
 namespace FolderListModelColumns {
   enum {icon, name, root_only, cols_num};
 }
 
 namespace FaxListModelColumns {
-  enum {name, fax_description, cols_num};
+  enum {fax_date, fax_description, name, cols_num};
 }
 
 std::pair<bool, std::string> FolderNameValidator::validate(const std::string& folder_name) {
@@ -213,7 +220,7 @@ gboolean FaxListManagerCB::fax_tree_view_motion_notify(GtkWidget*,
   int x, y;
   gtk_widget_get_pointer(GTK_WIDGET(instance_p->fax_tree_view_p), &x, &y);
 
-  if (x >= 0 && y >= 0){
+  if (x >= 0 && y >= 0) {
     bool result = gtk_tree_view_get_dest_row_at_pos(instance_p->fax_tree_view_p,
 						    x, y, 0, 0);
     if (result && !instance_p->fax_drag_source_enabled) {
@@ -281,7 +288,7 @@ void FaxListManagerCB::drag_data_received(GtkWidget*, GdkDragContext* context_p,
 	else instance_p->move_folder(&iter, drop_pos);
 	success = true;
       }
-      else write_error("Invalid iterator in FaxListDialogCB::drag_data_received()\n");
+      else write_error("Invalid iterator in FaxListManagerCB::drag_data_received()\n");
     }
   }
   gtk_drag_finish(context_p, success, false, time);
@@ -357,13 +364,13 @@ FaxListManager::FaxListManager(FaxListEnum::Mode mode_):
   gtk_tree_view_column_pack_start(column_p, renderer_p, false);
   gtk_tree_view_column_set_attributes(column_p, renderer_p,
 				      "pixbuf", FolderListModelColumns::icon,
-				      0);
+				      static_cast<void*>(0));
 
   renderer_p = gtk_cell_renderer_text_new();
   gtk_tree_view_column_pack_start(column_p, renderer_p, true);
   gtk_tree_view_column_set_attributes(column_p, renderer_p,
 				      "text", FolderListModelColumns::name,
-				      0);
+				      static_cast<void*>(0));
 
   // create the folder tree view
   folder_tree_view_p =
@@ -382,14 +389,14 @@ FaxListManager::FaxListManager(FaxListEnum::Mode mode_):
   // provide renderers for fax tree view, pack into the fax tree view columns
   // and connect to the fax tree model columns
   renderer_p = gtk_cell_renderer_text_new();
-  column_p = gtk_tree_view_column_new_with_attributes(gettext("Fax"), renderer_p,
-						      "text", FaxListModelColumns::name,
-						      0);
+  column_p = gtk_tree_view_column_new_with_attributes(gettext("Date"), renderer_p,
+						      "text", FaxListModelColumns::fax_date,
+						      static_cast<void*>(0));
   gtk_tree_view_append_column(fax_tree_view_p, column_p);
   renderer_p = gtk_cell_renderer_text_new();
   column_p = gtk_tree_view_column_new_with_attributes(gettext("Description"), renderer_p,
 						      "text", FaxListModelColumns::fax_description,
-						      0);
+						      static_cast<void*>(0));
   gtk_tree_view_append_column(fax_tree_view_p, column_p);
 
   selection_p = gtk_tree_view_get_selection(fax_tree_view_p);
@@ -468,7 +475,7 @@ void FaxListManager::populate_fax_list(void) {
     struct stat statinfo;
     std::list<std::string> dir_list;
 
-    // first populate dir_list with the directory names in $HOME/faxin
+    // first populate dir_list with the directory names in the faxin/faxsent subdirectories
     while ((direntry = readdir(dir_p)) != 0) {
       stat(direntry->d_name, &statinfo);
       if (S_ISDIR(statinfo.st_mode)
@@ -478,7 +485,7 @@ void FaxListManager::populate_fax_list(void) {
 	  && std::strcmp(direntry->d_name, ".")
 	  && std::strcmp(direntry->d_name, "..")
 	  && (mode == FaxListEnum::sent
-	      || std::strcmp(direntry->d_name, prog_config.receive_dirname))) {
+	      || std::strcmp(direntry->d_name, "current"))) {
 	dir_list.push_back(direntry->d_name);
       }
     }
@@ -497,8 +504,7 @@ void FaxListManager::populate_fax_list(void) {
     folder_row_to_fax_model_map.clear();
     folder_name_validator.clear();
 
-    // we always have an 'Inbox' and 'Sent box' row items to which children are attached
-    // get a list store row to insert the number and name in
+    // we always have an 'Inbox' and 'Sent box' row items
     GtkTreeIter base_folder_iter;
     gtk_tree_store_append(GTK_TREE_STORE(folder_tree_store_h.get()), &base_folder_iter, 0);
     std::string base_path;
@@ -509,7 +515,7 @@ void FaxListManager::populate_fax_list(void) {
 			 FolderListModelColumns::name, gettext("Inbox"),
 			 FolderListModelColumns::root_only, true,
 			 -1);
-      base_path +=  gettext("Inbox");
+      base_path += gettext("Inbox");
       folder_name_validator.insert_folder_name(gettext("Inbox"));
     }
     else {
@@ -538,8 +544,8 @@ void FaxListManager::populate_fax_list(void) {
 
     TreePathScopedHandle base_path_h(gtk_tree_model_get_path(folder_tree_store_h,
 							     &base_folder_iter));
-    TreeRowRefSharedHandle folder_row_ref_h(gtk_tree_row_reference_new(folder_tree_store_h,
-								       base_path_h));
+    TreeRowRefSharedHandle base_row_ref_h(gtk_tree_row_reference_new(folder_tree_store_h,
+								     base_path_h));
 
     TreePathScopedHandle trash_path_h(gtk_tree_model_get_path(folder_tree_store_h,
 							      &trash_folder_iter));
@@ -547,23 +553,26 @@ void FaxListManager::populate_fax_list(void) {
 									trash_path_h));
     
     PathToFolderRowMap path_to_row_map;
-    path_to_row_map.insert(PathToFolderRowMap::value_type(base_path, folder_row_ref_h));
+    path_to_row_map.insert(PathToFolderRowMap::value_type(base_path, base_row_ref_h));
     path_to_row_map.insert(PathToFolderRowMap::value_type(trash_path, trash_row_ref_h));
 
     // now create a fax tree model for the base and trash folders
-    GobjHandle<GtkTreeModel> base_model_h(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
-									    G_TYPE_STRING,
-									    G_TYPE_STRING)));
+    fax_base_model_h =
+      GobjHandle<GtkTreeModel>(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
+								 G_TYPE_STRING,
+								 G_TYPE_STRING,
+								 G_TYPE_STRING)));
     GobjHandle<GtkTreeModel> trash_model_h(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
+									     G_TYPE_STRING,
 									     G_TYPE_STRING,
 									     G_TYPE_STRING)));
-    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(base_model_h.get()),
+    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(fax_base_model_h.get()),
 					 FaxListModelColumns::name,
 					 GTK_SORT_ASCENDING);
     gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(trash_model_h.get()),
 					 FaxListModelColumns::name,
 					 GTK_SORT_ASCENDING);
-    folder_row_to_fax_model_map.insert(FolderRowToFaxModelMap::value_type(folder_row_ref_h, base_model_h));
+    folder_row_to_fax_model_map.insert(FolderRowToFaxModelMap::value_type(base_row_ref_h, fax_base_model_h));
     folder_row_to_fax_model_map.insert(FolderRowToFaxModelMap::value_type(trash_row_ref_h, trash_model_h));
 
     // get the paths (folders) stored in PathList
@@ -594,7 +603,11 @@ void FaxListManager::populate_fax_list(void) {
 
 	// this for loop can be time intensive as it reads from disk
 	// so clear any outstanding events in the main program loop
+	if (mode == FaxListEnum::received) fax_received_list_main_iteration = true;
+	else fax_sent_list_main_iteration = true;
 	while (gtk_events_pending()) gtk_main_iteration();
+	if (mode == FaxListEnum::received) fax_received_list_main_iteration = false;
+	else fax_sent_list_main_iteration = false;
 
 	line = "";
 	file.close();
@@ -633,7 +646,7 @@ void FaxListManager::populate_fax_list(void) {
 	file.open(filename.c_str(), std::ios::in);
 #endif
 
-	if (!g_utf8_validate(line.data(), line.size(), 0)) { // not valid UTF-8!
+	if (!Utf8::validate(line)) { // not valid UTF-8!
 	  write_error("Invalid UTF-8 string in folder path name in FaxListManager::populate_fax_list()\n");
 	  line = "";
 	}
@@ -692,7 +705,7 @@ bool FaxListManager::get_folders(PathToFolderRowMap& path_to_row_map, const std:
   std::vector<std::string> child_rowlist;
   std::string remaining_path(line);
 
-  if (!g_utf8_validate(line.data(), line.size(), 0)) { // not valid UTF-8!
+  if (!Utf8::validate(line)) { // not valid UTF-8!
     write_error("Invalid UTF-8 string in folder path name in FaxListManager::get_folders()\n");
     remaining_path = "";
     return_val = false;
@@ -741,6 +754,7 @@ bool FaxListManager::get_folders(PathToFolderRowMap& path_to_row_map, const std:
       path_to_row_map.insert(PathToFolderRowMap::value_type(pathname, row_ref_h));
       // create a fax tree model for the folder
       GobjHandle<GtkTreeModel> model_h(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
+									 G_TYPE_STRING,
 									 G_TYPE_STRING,
 									 G_TYPE_STRING)));
       gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model_h.get()),
@@ -772,6 +786,7 @@ bool FaxListManager::get_folders(PathToFolderRowMap& path_to_row_map, const std:
 	path_to_row_map.insert(PathToFolderRowMap::value_type(pathname, row_ref_h));
 	// create a fax tree model for the folder
 	GobjHandle<GtkTreeModel> model_h(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
+									   G_TYPE_STRING,
 									   G_TYPE_STRING,
 									   G_TYPE_STRING)));
 	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model_h.get()),
@@ -812,6 +827,7 @@ bool FaxListManager::get_folders(PathToFolderRowMap& path_to_row_map, const std:
 	path_to_row_map.insert(PathToFolderRowMap::value_type(pathname, row_ref_h));
 	// create a fax tree model for the folder
 	GobjHandle<GtkTreeModel> model_h(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
+									   G_TYPE_STRING,
 									   G_TYPE_STRING,
 									   G_TYPE_STRING)));
 	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model_h.get()),
@@ -837,13 +853,13 @@ void FaxListManager::insert_fax_on_populate(const PathToFolderRowMap& path_to_ro
   
   PathToFolderRowMap::const_iterator path_map_iter = path_to_row_map.find(fax_path);
   if (path_map_iter == path_to_row_map.end()) {
-    write_error("Mapping error (no folder row) in FaxListManager::insert_fax()\n");
+    write_error("Mapping error (no folder row) in FaxListManager::insert_fax_on_populate()\n");
   }
   else {
     FolderRowToFaxModelMap::const_iterator row_map_iter =
       folder_row_to_fax_model_map.find(path_map_iter->second);
     if (row_map_iter == folder_row_to_fax_model_map.end()) {
-      write_error("Mapping error in FaxListManager::insert_fax()\n");
+      write_error("Mapping error in FaxListManager::insert_fax_on_populate()\n");
     }
     else {
 	
@@ -854,27 +870,80 @@ void FaxListManager::insert_fax_on_populate(const PathToFolderRowMap& path_to_ro
       gtk_list_store_set(GTK_LIST_STORE(fax_model_h.get()), &row_iter,
 			 FaxListModelColumns::name, fax_name.c_str(),
 			 -1);
-    
+
+      // convert faxname to locale date-time format for fax date
+      std::string date(convert_faxname_to_date(fax_name));
+  
+      try {
+	gtk_list_store_set(GTK_LIST_STORE(fax_model_h.get()), &row_iter,
+			   FaxListModelColumns::fax_date,
+			   Utf8::locale_to_utf8(date).c_str(),
+			   -1);
+      }
+      catch (Utf8::ConversionError&) {
+	write_error("UTF-8 conversion error in FaxListManager::insert_fax_on_populate()\n");
+      }
+
       // now see if there is a description
       std::string line;
       if (file) {
 	while (std::getline(file, line) && line.empty());
       }
       if (!line.empty()) {
-	try {
+	if (Utf8::validate(line)) {
+	  // no filename to UTF-8 conversion is required as the fax description is
+	  // stored in UTF-8 (since version 3.0.7 of efax-gtk)
 	  gtk_list_store_set(GTK_LIST_STORE(fax_model_h.get()), &row_iter,
 			     FaxListModelColumns::fax_description,
-			     Utf8::locale_to_utf8(line).c_str(),
+			     line.c_str(),
 			     -1);
 	}
-	catch (Utf8::ConversionError&) {
-	  write_error("UTF-8 conversion error in FaxListDialog::insert_fax_on_populate()\n");
+	else {
+	  //write_error("Invalid UTF-8 in fax description in FaxListManager::insert_fax_on_populate()\n");
 	}
       }
     }
   }
 }
 
+void FaxListManager::insert_new_fax_in_base(const std::string& fax_name,
+					    const std::string& description) {
+
+  GtkTreeIter row_iter;
+  gtk_list_store_append(GTK_LIST_STORE(fax_base_model_h.get()), &row_iter);
+  gtk_list_store_set(GTK_LIST_STORE(fax_base_model_h.get()), &row_iter,
+		     FaxListModelColumns::name, fax_name.c_str(),
+		     -1);
+
+  // convert faxname to locale date-time format for fax date
+  std::string date(convert_faxname_to_date(fax_name));
+  
+  try {
+    gtk_list_store_set(GTK_LIST_STORE(fax_base_model_h.get()), &row_iter,
+		       FaxListModelColumns::fax_date,
+		       Utf8::locale_to_utf8(date).c_str(),
+		       -1);
+  }
+  catch (Utf8::ConversionError&) {
+    write_error("UTF-8 conversion error in FaxListManager::insert_new_fax_in_base()\n");
+  }
+
+  // now insert any description
+  if (!description.empty()) {
+    // there is no need to convert the fax description to UTF-8 as it is
+    // already in that codeset - but let's check
+    if (Utf8::validate(description)) {
+      gtk_list_store_set(GTK_LIST_STORE(fax_base_model_h.get()), &row_iter,
+			 FaxListModelColumns::fax_description,
+			 description.c_str(),
+			 -1);
+    }
+    else {
+      write_error("Invalid UTF-8 in fax description in FaxListManager::insert_new_fax_in_base()\n");
+    }
+  }
+}
+
 void FaxListManager::move_fax(const GtkTreeIter* dest_folder_row_iter_p) {
 
   // check pre-conditions
@@ -906,26 +975,33 @@ void FaxListManager::move_fax(const GtkTreeIter* dest_folder_row_iter_p) {
       if (old_fax_path_h.get() &&
 	  gtk_tree_model_get_iter(source_model_p, &old_fax_row_iter, old_fax_path_h)) {
 	
-	gchar* name_p = 0;
+	gchar* date_p = 0;
 	gchar* description_p = 0;
+	gchar* name_p = 0;
 	gtk_tree_model_get(source_model_p, &old_fax_row_iter,
-			   FaxListModelColumns::name, &name_p,
+			   FaxListModelColumns::fax_date, &date_p,
 			   FaxListModelColumns::fax_description, &description_p,
+			   FaxListModelColumns::name, &name_p,
 			   -1);
-	std::string faxname;
-	if (name_p) faxname = name_p;
 	std::string description;
 	if (description_p) description = description_p;
+	std::string date;
+	if (date_p) date = date_p;
+	std::string faxname;
+	if (name_p) faxname = name_p;
+	
 	// we have not placed the gchar* strings given by gtk_tree_model_get()
 	// in a GcharScopedHandle object so we need to free them by hand
-	g_free(name_p);
+	g_free(date_p);
 	g_free(description_p);
+	g_free(name_p);
 
 	GtkTreeIter new_fax_row_iter;
 	gtk_list_store_append(GTK_LIST_STORE(dest_model_h.get()), &new_fax_row_iter);
 	gtk_list_store_set(GTK_LIST_STORE(dest_model_h.get()), &new_fax_row_iter,
-			   FaxListModelColumns::name, faxname.c_str(),
+			   FaxListModelColumns::fax_date, date.c_str(),
 			   FaxListModelColumns::fax_description, description.c_str(),
+			   FaxListModelColumns::name, faxname.c_str(),
 			   -1);
 	gtk_list_store_remove(GTK_LIST_STORE(gtk_tree_view_get_model(fax_tree_view_p)),
 			      &old_fax_row_iter);
@@ -1361,6 +1437,7 @@ void FaxListManager::make_folder(const std::string& folder_name, bool test_valid
 							path_h));
     // create a fax tree model for the folder
     GobjHandle<GtkTreeModel> model_h(GTK_TREE_MODEL(gtk_list_store_new(FaxListModelColumns::cols_num,
+								       G_TYPE_STRING,
 								       G_TYPE_STRING,
 								       G_TYPE_STRING)));
     gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model_h.get()),
@@ -1489,12 +1566,6 @@ void FaxListManager::delete_fax(void) {
   }
 }
 
-void FaxListManager::refresh(void) {
-  
-  write_path();
-  populate_fax_list();
-}
-
 void FaxListManager::write_path(void) {
 
   std::string dir(prog_config.working_dir);
@@ -1622,21 +1693,15 @@ void FaxListManager::describe_fax(const std::string& description) {
 
       filename += "/Description";
       std::ofstream file(filename.c_str(), std::ios::out);
-      // this try()/catch() block is ultra-cautious - something has gone seriously
-      // wrong with the program if the UTF-8 conversion fails since the source is
-      // a Gtk::Entry object
-      try {
-	if (file) file << Utf8::locale_from_utf8(description);
+      // do not convert the description to the locale codeset - save it in UTF-8
+      // (with effect from version 3.0.7)
+      if (file) file << description;
     
-	else {
-	  std::string msg("Can't open file ");
-	  msg += filename;
-	  msg += '\n';
-	  write_error(msg.c_str());
-	}
-      }
-      catch (Utf8::ConversionError&) {
-	write_error("UTF-8 conversion error in FaxListDialog::describe_fax()\n");
+      else {
+	std::string msg("Can't open file ");
+	msg += filename;
+	msg += '\n';
+	write_error(msg.c_str());
       }
     }
   }
@@ -1737,3 +1802,24 @@ bool FaxListManager::is_folder_selected(void) {
   GtkTreeSelection* selection_p = gtk_tree_view_get_selection(folder_tree_view_p);
   return gtk_tree_selection_get_selected(selection_p, 0, 0);
 }
+
+std::string FaxListManager::convert_faxname_to_date(const std::string& faxname) {
+
+  struct std::tm t;
+  char str[150];
+  
+  // init
+  std::memset(&t, 0, sizeof(struct std::tm));
+  std::memset(str, 0, sizeof(str));
+  // convert date --> struct tm
+  if (faxname.size() > 12) { // the "new" format for creating fax directories
+    strptime(faxname.c_str(), "%Y%m%d%H%M%S", &t);
+  }
+  else {                     // the "old" format for creating fax directories
+    strptime(faxname.c_str(), "%y%m%d%H%M%S", &t);
+  }
+
+  // convert tm to date and time in current locale
+  std::strftime(str, sizeof(str), "%x %H:%M", &t);
+  return std::string(str);
+}
