@@ -250,11 +250,8 @@ const char* HelpDialog::get_receiving_help(void) {
 		 "\n"
 		 "Received faxes in tiffg3 format (one file for each page) are placed in "
 		 "a directory in the $HOME/faxin directory, which has a name derived "
-		 "from the year, month, day, hour and seconds when the relevant \"Answer "
-		 "call\", \"Take over call\" or \"Standy\" button was pressed.  (Where in "
-		 "standby mode after a fax has been received, any further fax will "
-		 "derive its name from the time when receipt of the last received fax "
-		 "has been completed and the program goes back into standby mode.)\n"
+		 "from the year, month, day, hour and seconds when reception of the fax "
+		 "was completed, and is the fax ID number.\n"
 		 "\n"
 		 "Received faxes can be printed, viewed, described and managed using the "
 		 "built in fax list facility.  This can be brought up from the "
@@ -264,7 +261,7 @@ const char* HelpDialog::get_receiving_help(void) {
 		 "When a fax is received, a pop-up dialog can also be set to appear (go "
 		 "to the Settings dialog to do this).  In the settings dialog you can "
 		 "also specify a program to be executed when a fax is received.  The "
-		 "number of the fax is passed as the first (and only) argument to the "
+		 "fax ID number is passed as the first (and only) argument to the "
 		 "program, which enables the program to find the fax in $HOME/faxin.  "
 		 "The distribution contains two executable scripts, mail_fax and "
 		 "print_fax, which can be used to e-mail a fax or print a fax to a user "
@@ -317,8 +314,12 @@ const char* HelpDialog::get_fax_list_help(void) {
 		 "list (or subsequently amended) by pressing the relevant button -- this "
 		 "will enable faxes to be more easily identified.\n"
 		 "\n"
-		 "To refresh a fax list (say, to update with any faxes received or sent "
-		 "since the fax list was brought up), press the refresh button.\n");
+		 "The received faxes list will show, at the far right of the tool bar, "
+		 "the number of faxes received since the program was last started.  If "
+		 "efax-gtk is in receive standby mode, the `tooltips' for the program's "
+		 "icon in the system tray will also indicate this number.  The count can "
+		 "be reset to 0 without restarting the program by pressing the reset "
+		 "button in the received faxes list.\n");
 }
 
 const char* HelpDialog::get_settings_help(void) {
