@@ -11,6 +11,7 @@ COPYING distributed with the source files.
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <dirent.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <errno.h>
@@ -20,7 +21,10 @@ COPYING distributed with the source files.
 #include <iostream>
 #include <iomanip>
 #include <string>
+#include <vector>
+#include <algorithm>
 #include <cstring>
+#include <ctime>
 
 #include <gtk/gtkmain.h>
 #include <gtk/gtkversion.h>
@@ -55,16 +59,18 @@ COPYING distributed with the source files.
 
 #ifdef EFAX_GTK_USE_ICON_THEME
 #include <gtk/gtkicontheme.h>
-#include "icon_info_handle.h"
+#include "utils/icon_info_handle.h"
 #endif
 
 #include "mainwindow.h"
 #include "dialogs.h"
-#include "mutex.h"
 #include "window_icon.h"
-#include "utf8_utils.h"
-#include "gobj_handle.h"
-#include "fdstream/fdstream.h"
+#include "utils/mutex.h"
+#include "utils/utf8_utils.h"
+#include "utils/gobj_handle.h"
+#include "utils/fdstream.h"
+#include "utils/pipes.h"
+#include "utils/notifier.h"
 
 #ifdef ENABLE_NLS
 #include <locale.h>
@@ -80,18 +86,25 @@ extern "C" pid_t getpgid(pid_t);
 Prog_config prog_config;
 
 bool is_program_running(void);
-
 bool get_prog_parm(const char*, std::string&, std::string&, std::string(*)(const std::string&));
 
 inline bool get_prog_parm(const char* name, std::string& line, std::string& result) {
   return get_prog_parm(name, line, result, Utf8::locale_to_utf8);
 }
 
+inline bool not_ascii_char(char c) {
+  if (static_cast<signed char>(c) < 0) return true;
+  return false;
+}
+
 void get_fonts(void);
 void get_window_icon(void);
 int mkdir_with_parent(const std::string&);
 bool is_arg(const char*, int, char*[]);
 extern "C" void atexit_cleanup(void);
+void clean_up_receive_dir(void);
+bool is_ascii(const std::string&);
+
 
 int main(int argc, char* argv[]) {
 
@@ -149,7 +162,9 @@ int main(int argc, char* argv[]) {
   }
 
   // although we use Thread::Thread and Thread::Mutex classes based on pthreads,
-  // we need tocall g_thread_init() in order to make glib thread safe
+  // we need to call g_thread_init() in order to make glib thread safe
+  // (we do not use the GTK global thread lock, so there is no need to
+  // call gdk_threads_init())
   g_thread_init(0);
   prog_config.mutex_p = new Thread::Mutex;
   
@@ -201,12 +216,30 @@ int main(int argc, char* argv[]) {
       }
       const char* filename = 0;
       if (optind < argc) filename = argv[optind];
-      
-      MainWindow mainwindow(messages, start_hidden, start_in_standby, filename);
 
-      // everything is set up
-      // now enter the main program loop
-      gtk_main();
+      // this program cleans up for closure by ordinary user action, and
+      // atexit_cleanup() in main.cpp caters for cases where the X session
+      // has brought about the closure of this program.  It is also possible
+      // that no clean-up at all will occur if the program is terminated on,
+      // say, a power failure.  So call the clean_up_receive_dir() function
+      // now to deal with that case.
+      clean_up_receive_dir();
+
+      try {
+	MainWindow mainwindow(messages, start_hidden, start_in_standby, filename);
+
+	// everything is set up
+	// now enter the main program loop
+	gtk_main();
+      }
+      catch (PipeError& error) {
+	std::cerr << error.error_message;
+	std::exit(PIPE_ERROR);
+      }
+      catch (NotifierError& error) {
+	std::cerr << error.error_message;
+	std::exit(NOTIFIER_ERROR);
+      }
     }
   }
 #ifdef HAVE_NOTIFY_STARTUP
@@ -334,12 +367,8 @@ std::string configure_prog(bool reread) {
       prog_config.fax_received_prog = "";
       prog_config.logfile_name = "";
     }
-    // if we are setting up for first time, initialise prog_config.receive_dirname
-    // and prog_config.efax_controller_child_pid;
-    else {
-      *prog_config.receive_dirname = 0;
-      prog_config.efax_controller_child_pid = 0;
-    }
+    // if we are setting up for first time, initialise prog_config.efax_controller_child_pid;
+    else prog_config.efax_controller_child_pid = 0;
 
 // now extract settings from file
 
@@ -360,6 +389,7 @@ std::string configure_prog(bool reread) {
     std::string sock_other_addresses;
     std::string fax_received_popup;
     std::string fax_received_exec;
+    std::string addr_in_header;
     std::string work_subdir;
     
     while (std::getline(filein, file_read)) {
@@ -450,6 +480,9 @@ std::string configure_prog(bool reread) {
 	else if (get_prog_parm("LOG_FILE:", file_read, prog_config.logfile_name,
 			       Utf8::filename_to_utf8));
 
+	// look for "ADDR_IN_HEADER:"
+	else if (get_prog_parm("ADDR_IN_HEADER:", file_read, addr_in_header));
+
 	// look for "WORK_SUBDIR:"
 	else if (get_prog_parm("WORK_SUBDIR:", file_read, work_subdir,
 			       Utf8::filename_to_utf8));
@@ -465,9 +498,20 @@ std::string configure_prog(bool reread) {
     prog_config.parms.push_back("-vin");  // stdin -- information and negotiations
     //prog_config.parms.push_back("-vina");  // this will also report the parameters passed to efax
                                              // uncomment it and comment out preceding line for debugging
+
+    prog_config.parms.push_back("-u");    // force use of UTF-8 to send messages to efax-gtk
     
     std::string temp;
 
+    if (!is_ascii(prog_config.my_name)) {
+      prog_config.my_name = "";
+      return_val += gettext("Invalid user name specified - it must be in plain\n"
+			    "ASCII characters.  No user name will be shown on\n"
+			    "the fax top header line, but don't worry as the\n"
+			    "fax station number will always be given on the\n"
+			    "top header\n");
+    }
+
     if (!prog_config.my_number.empty()) {
       temp = "-l";
       temp += prog_config.my_number;
@@ -735,6 +779,10 @@ std::string configure_prog(bool reread) {
     if (!temp.compare("yes")) prog_config.fax_received_exec = true;
     else prog_config.fax_received_exec = false;
 
+    temp = Utf8::to_lower(addr_in_header);
+    if (!temp.compare("no")) prog_config.addr_in_header = false;
+    else prog_config.addr_in_header = true;
+
     if (!reread) {  // prog_config.working_dir and prog_config.GPL_flag are not
                     // affected by the settings dialog and so not re-configurable
                     // after the program has initialised itself for the first time
@@ -916,7 +964,7 @@ bool is_program_running(void) {
     else {   // the lockfile already exists - see if it is stale
              // it shouldn't ever be stale with the other clean up provision
              // in this program, but nonetheless ...
-      ifstream existing_lock(lockfile.c_str(), ios::in);
+      std::ifstream existing_lock(lockfile.c_str(), std::ios::in);
       if(!existing_lock) { // can't open lock file
 	std::cerr << "Not able to access existing lockfile " << lockfile << std::endl;
 	// get rid of the old lockfile if we can
@@ -966,12 +1014,21 @@ bool is_program_running(void) {
   return return_val;
 }
 
+bool is_ascii(const std::string& text) {
+  
+  std::string::const_iterator end_iter = text.end();
+  if (std::find_if(text.begin(), end_iter, not_ascii_char) == end_iter) {
+    return true;
+  }
+  return false;
+}
+
 void atexit_cleanup(void) {
 
   // this does any essential clean up, which is always required to make the program
-  // restart correctly.  It is only relevant to a case where the program has closed
+  // restart correctly.  It is mainly relevant to a case where the program has closed
   // down because X server connection has closed - where the program is closed by direct
-  // user input to the program, then the program cleans up elsewhere
+  // user input to the program, then the program generally cleans up elsewhere
 
   // check we are the process group leader (we should be as setpgid() was
   // called in main()) and if so kill any stray children, such as an efax
@@ -1000,15 +1057,142 @@ void atexit_cleanup(void) {
   fifo_name += PIPE_NAME;
   unlink(fifo_name.c_str());
 
-  // delete receive_dirname if it is empty
-  if (*prog_config.receive_dirname && !prog_config.working_dir.empty()) {
-    std::string full_dirname(prog_config.working_dir);
-    full_dirname += "/faxin/";
-    full_dirname += prog_config.receive_dirname;
-    rmdir(full_dirname.c_str()); // only deletes the directory if it is empty
-  }
+  // clean up the receive_directory if necessary
+  clean_up_receive_dir();
 
   // call up a shell to dispense with any stray viewing files
   // (these may exist if the program is exited when a fax is still being viewed)
+  // whether this executes or not is pretty optional so whilst I would normally
+  // avoid calling system(), it is acceptable in this limited context (IMO)!
   std::system("rm -f /tmp/efax-gtk-view.*");
 }
+
+void clean_up_receive_dir(void) {
+
+  if (!prog_config.working_dir.empty()) {
+    std::string full_dirname(prog_config.working_dir);
+    full_dirname += "/faxin/current";
+    int result = rmdir(full_dirname.c_str());
+    if (result == -1 && errno == ENOTEMPTY) { // we must be in the middle of receiving a fax
+                                              // when the program ended - save what we can
+
+      std::cerr << "The program closed when receiving a fax\n"
+	           "Attempting to save what we can"
+		<< std::endl;
+
+      char faxname[18];
+      *faxname = 0;
+
+      // get a time value to create the directory name into which the fax is to be saved
+      struct std::tm* time_p;
+      std::time_t time_count;
+    
+      std::time(&time_count);
+      time_p = std::localtime(&time_count);
+
+      std::string new_dirname(prog_config.working_dir);
+      new_dirname += "/faxin/";
+
+      const char format[] = "%Y%m%d%H%M%S";
+      std::strftime(faxname, sizeof(faxname), format, time_p);
+      std::string temp(new_dirname);
+      temp += faxname;
+
+      // check whether directory already exists or can't be created
+      int count;
+      for (count = 0; count < 4 && mkdir(temp.c_str(), S_IRUSR | S_IWUSR | S_IXUSR); count++) {
+	// it is not worth checking for a short sleep because of a signal as we will
+	// repeat the attempt four times anyway
+	sleep(1); // wait a second to get a different time
+	std::time(&time_count);
+	time_p = std::localtime(&time_count);
+	std::strftime(faxname, sizeof(faxname), format, time_p);
+	temp = new_dirname + faxname;
+      }
+      if (count == 4) {
+	std::cerr << "Can't create directory to save received fax on clean up.\n"
+	             "This fax will not be saved!"
+		  << std::endl;
+	*faxname = 0;
+      }
+
+      else { // move the faxes into the new directory
+
+	// make new_dirname the same as the directory name for the saved faxes
+	// that we have just created
+	new_dirname += faxname;
+
+	std::string old_dirname(prog_config.working_dir);
+	old_dirname += "/faxin/current";
+
+	std::vector<std::string> filelist;
+	struct dirent* direntry;
+	struct stat statinfo;
+
+	DIR* dir_p;
+	if ((dir_p = opendir(old_dirname.c_str())) == 0) {
+	  std::cerr << "Can't open received fax directory.\n"
+	               "This fax will not be saved!"
+		    << std::endl;
+	}
+
+	else {
+	  chdir(old_dirname.c_str());
+	  while ((direntry = readdir(dir_p)) != 0) {
+	    stat(direntry->d_name, &statinfo);
+	    if (S_ISREG(statinfo.st_mode)) {
+	      filelist.push_back(direntry->d_name);
+	    }
+	  }
+
+	  closedir(dir_p);
+
+	  bool failure = false;
+	  std::vector<std::string>::const_iterator iter;
+	  std::string old_filename;
+	  std::string new_filename;
+
+	  for (iter = filelist.begin(); iter != filelist.end(); ++iter) {
+	    old_filename = old_dirname;
+	    old_filename += '/';
+	    old_filename += *iter;
+	    if (!iter->substr(0, std::strlen("current.")).compare("current.")) {
+	      new_filename = new_dirname;
+	      new_filename += '/';
+	      new_filename += faxname;
+	      new_filename += iter->substr(std::strlen("current"));  // add the suffix (".001" etc.)
+	      if (link(old_filename.c_str(), new_filename.c_str())) {
+		failure = true;
+		break;
+	      }
+	    }
+	  }
+	  if (failure) {
+	    std::cerr << "Cannot save all of the received fax.\n"
+	                 "All or part of the fax will be missing!"
+		      << std::endl;
+	  }
+	  else {
+	    for (iter = filelist.begin(); iter != filelist.end(); ++iter) {
+	      old_filename = old_dirname;
+	      old_filename += '/';
+	      old_filename += *iter;
+	      unlink(old_filename.c_str());
+	    }
+	    if (rmdir(old_dirname.c_str())) {
+	      std::cerr << "Can't delete directory " << old_dirname
+			<< "\nThe contents should have been moved to "
+			<< new_dirname
+			<< "\nand it should now be empty"
+			<< std::endl;
+	    }
+	  }	
+	}
+
+	// reset current directory
+	std::string temp(prog_config.working_dir + "/faxin");
+	chdir(temp.c_str());
+      }
+    }
+  }
+}
