@@ -72,7 +72,7 @@ COPYING distributed with the source files.
 
 #ifdef EFAX_GTK_USE_ICON_THEME
 #include <gtk/gtkicontheme.h>
-#include "icon_info_handle.h"
+#include "utils/icon_info_handle.h"
 #endif
 
 #include "mainwindow.h"
@@ -82,10 +82,9 @@ COPYING distributed with the source files.
 #include "settings.h"
 #include "socket_notify.h"
 #include "menu_icons.h"
-#include "shared_handle.h"
-#include "utf8_utils.h"
-#include "thread.h"
-#include "fdstream/fdstream.h"
+#include "utils/shared_handle.h"
+#include "utils/thread.h"
+#include "utils/fdstream.h"
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -306,7 +305,7 @@ MainWindow::MainWindow(const std::string& messages,bool start_hidden,
 
   file_button_p = gtk_radio_button_new_with_label(0, gettext("File  "));
   socket_button_p = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(file_button_p),
-								gettext("Socket"));
+								gettext("Socket "));
 
   GtkWidget* fax_method_label_p = gtk_label_new(gettext("Fax entry method: "));
   single_file_button_p = gtk_button_new_with_label(gettext("Single file"));
@@ -753,6 +752,8 @@ MainWindow::MainWindow(const std::string& messages,bool start_hidden,
   tray_item.menu_item_chosen.connect(sigc::mem_fun(*this,
 					  &MainWindow::tray_icon_menu_slot));
   tray_item.get_state.connect(sigc::mem_fun(efax_controller, &EfaxController::get_state));
+  tray_item.get_new_fax_count.connect(sigc::mem_fun(efax_controller,
+						    &EfaxController::get_count));
   efax_controller.write_state.connect(sigc::mem_fun(tray_item, &TrayItem::set_tooltip_slot));
 
   present_window_notify.connect(sigc::mem_fun(*this, &MainWindow::present_window_slot));
@@ -876,37 +877,48 @@ void MainWindow::sendfax_slot(void) {
 
   gtk_window_present(get_win());
 
-  fax_item.is_socket_file = false;
-
-  std::string files;
-  try {
-    files = Utf8::filename_from_utf8(gtk_entry_get_text(GTK_ENTRY(file_entry_p)));
-  }
-  catch (Utf8::ConversionError&) {
-    write_error("UTF-8 conversion error in sendfax_slot()\n");
-    beep();
-    return;
+  std::string files(gtk_entry_get_text(GTK_ENTRY(file_entry_p)));
+  // if this is a fax received from the socket and entered via the socket faxes
+  // list, then "*** " will form the first part of the "file" displayed in the file
+  // entry box - so test for it to see if we are sending a file received via the
+  // socket rather than a regular file
+  if (!files.empty() && files.substr(0,4) == std::string("*** ")) { 
+    fax_item.is_socket_file = true;
+    fax_item.file_list.push_back(selected_socket_list_file);
   }
 
-  const char* separators = ",;";
-  // split the files string into separate file names
-  std::string::size_type pos1, pos2;
-  pos1 = files.find_first_not_of(separators);
-  while (pos1 != std::string::npos) {
-    pos2 = files.find_first_of(separators, pos1);
-    if (pos2 != std::string::npos) {
-      std::string file_item(files.substr(pos1, pos2 - pos1));
-      strip(file_item);
-      fax_item.file_list.push_back(file_item);
-      pos1 = files.find_first_not_of(separators, pos2);
+  else {
+    fax_item.is_socket_file = false;
+    // convert the contents of files to locale filename codeset if necessary
+    try {
+      files = Utf8::filename_from_utf8(files);
     }
-    else {
-      std::string file_item(files.substr(pos1));
-      strip(file_item);
-      fax_item.file_list.push_back(file_item);
-      pos1 = std::string::npos;
+    catch (Utf8::ConversionError&) {
+      write_error("UTF-8 conversion error in sendfax_slot()\n");
+      beep();
+      return;
     }
-  }  
+
+    const char* separators = ",;";
+    // split the files string into separate file names
+    std::string::size_type pos1, pos2;
+    pos1 = files.find_first_not_of(separators);
+    while (pos1 != std::string::npos) {
+      pos2 = files.find_first_of(separators, pos1);
+      if (pos2 != std::string::npos) {
+	std::string file_item(files.substr(pos1, pos2 - pos1));
+	strip(file_item);
+	fax_item.file_list.push_back(file_item);
+	pos1 = files.find_first_not_of(separators, pos2);
+      }
+      else {
+	std::string file_item(files.substr(pos1));
+	strip(file_item);
+	fax_item.file_list.push_back(file_item);
+	pos1 = std::string::npos;
+      }
+    }  
+  }
 
   if (!prog_config.found_rcfile) {
     text_window.write_red_slot("Can't send fax -- no efax-gtkrc configuration file found\n\n");
@@ -922,22 +934,7 @@ void MainWindow::sendfax_slot(void) {
 
   else if (fax_item.file_list.empty()) beep();
 
-  else {
-
-    // if this is a fax received from the socket and entered via the socket faxes
-    // list, then "*** " will form the first part of the "file" displayed in the file
-    // entry box - so test for it to see if we are sending a file received via the
-    // socket rather than a regular file
-    if (fax_item.file_list.size() > 0
-	&& fax_item.file_list[0].substr(0,4) == std::string("*** ")) { 
-
-      fax_item.file_list.clear();
-      fax_item.file_list.push_back(selected_socket_list_file);
-      fax_item.is_socket_file = true;
-    }
-
-    efax_controller.sendfax(fax_item);
-  }
+  else efax_controller.sendfax(fax_item);
 }
 
 void MainWindow::receive_impl(EfaxController::State mode) {
@@ -974,21 +971,39 @@ void MainWindow::set_files_slot(const std::string& files) {
 void MainWindow::fax_list_impl(FaxListEnum::Mode mode) {
 
   if (mode == FaxListEnum::received) {
-    if (!FaxListDialog::get_is_fax_received_list()) {
-      received_fax_list_p = new FaxListDialog(mode, standard_size);
-    }
-    else {
-      received_fax_list_p->refresh();
-      gtk_window_present(received_fax_list_p->get_win());
+
+    // because FaxListManager::populate_fax_list() calls gtk_main_iteration()
+    // we could get back here again before the constructor of FaxListDialog
+    // has been entered, so check the guard in FaxListManager
+    if (!FaxListManager::is_fax_received_list_main_iteration()) {
+
+      if (!FaxListDialog::get_is_fax_received_list()) {
+	received_fax_list_p = new FaxListDialog(mode, standard_size);
+	efax_controller.fax_received_notify.connect(sigc::mem_fun(*received_fax_list_p,
+						       &FaxListDialog::insert_new_fax_slot));
+	received_fax_list_p->get_new_fax_count_sig.connect(sigc::mem_fun(efax_controller,
+						       &EfaxController::get_count));
+	received_fax_list_p->reset_sig.connect(sigc::mem_fun(efax_controller,
+						       &EfaxController::reset_count));
+	// now display the correct number of new received faxes
+	received_fax_list_p->display_new_fax_count();
+      }
+      else gtk_window_present(received_fax_list_p->get_win());
     }
   }
   else if (mode == FaxListEnum::sent) {
-    if (!FaxListDialog::get_is_fax_sent_list()) {
-      sent_fax_list_p = new FaxListDialog(mode, standard_size);
-    }
-    else {
-      sent_fax_list_p->refresh();
-      gtk_window_present(sent_fax_list_p->get_win());
+
+    // because FaxListManager::populate_fax_list() calls gtk_main_iteration()
+    // we could get back here again before the constructor of FaxListDialog
+    // has been entered, so check the guard in FaxListManager
+    if (!FaxListManager::is_fax_sent_list_main_iteration()) {
+
+      if (!FaxListDialog::get_is_fax_sent_list()) {
+	sent_fax_list_p = new FaxListDialog(mode, standard_size);
+	efax_controller.fax_sent_notify.connect(sigc::mem_fun(*sent_fax_list_p,
+						       &FaxListDialog::insert_new_fax_slot));
+      }
+      else gtk_window_present(sent_fax_list_p->get_win());
     }
   }
   // there is no memory leak -- FaxListDialog is modeless and will delete its own memory
@@ -1081,7 +1096,7 @@ void MainWindow::fax_to_send_dialog(const std::pair<std::string, unsigned int>&
   // is self-owning and will delete itself when it is closed
 }
 
-void MainWindow::fax_received_notify_slot(void) {
+void MainWindow::fax_received_notify_slot(const std::pair<std::string, std::string>& fax_info) {
 
   if (prog_config.fax_received_exec
       && !prog_config.fax_received_prog.empty()) {
@@ -1097,8 +1112,10 @@ void MainWindow::fax_received_notify_slot(void) {
       write_error("UTF-8 conversion error in MainWindow::fax_received_notify_slot()\n");
       cmd.assign(prog_config.fax_received_prog);
     }
-    // we can use the raw output of cmd.c_str() here - it will remain valid at the execlp() call
+    // we can use the raw output of cmd.c_str() and fax_info.first.c_str() here
+    // - they will remain valid at the execlp() call
     const char* prog_name = cmd.c_str();
+    const char* fax_name = fax_info.first.c_str();
 
     pid_t pid = fork();
     
@@ -1111,7 +1128,10 @@ void MainWindow::fax_received_notify_slot(void) {
       // now we have forked, we can connect MainWindow::error_pipe to stderr
       connect_to_stderr();
 
-      execlp (prog_name, prog_name, prog_config.receive_dirname, 0);
+      // this function requires a null pointer as its last argument but as its
+      // latter arguments are passed as an elipsis we need to do an explicit cast
+      // for 64 bit systems
+      execlp(prog_name, prog_name, fax_name, static_cast<void*>(0));
 
       // if we reached this point, then the execlp() call must have failed
       // report error and end process - use _exit() and not exit()
@@ -1344,7 +1364,8 @@ void MainWindow::translations_impl(void) {
   message += gettext("Hebrew - Assaf Gillat\n");
   message += gettext("Greek - Hellenic Linux Users Group\n");
   message += gettext("Albanian - Besnik Bleta\n");
-  message += gettext("Hungarian - Gergely Szakats");
+  message += gettext("Hungarian - Gergely Szakats\n");
+  message += gettext("Simplified Chinese - Kite Lau");
   new InfoDialog(message.c_str(), gettext("efax-gtk: Translations"),
 		 GTK_MESSAGE_INFO, get_win());
 
@@ -1475,7 +1496,7 @@ void MainWindow::style_set_impl(void) {
   gtk_widget_set_size_request(stop_button_p, width, height);
 
   // now set the status line style
-  status_line.set_status_line(gtk_widget_get_style(GTK_WIDGET(get_win())), height - 6);
+  status_line.set_status_line(height - 6);
 }
 
 bool MainWindow::draw_fax_from_socket_notifier(void) {
@@ -1529,21 +1550,26 @@ bool MainWindow::draw_fax_from_socket_notifier(void) {
       red.blue = 0;
 
       GdkColormap* colormap_p = gtk_widget_get_default_colormap();
-      gdk_colormap_alloc_color(colormap_p, &red, false, true);
-      GobjHandle<GdkGC> red_gc_h(gdk_gc_new(drawable_p));
-      gdk_gc_set_foreground(red_gc_h, &red);
+      if (!gdk_colormap_alloc_color(colormap_p, &red, false, true)) {
+	write_error("Error allocating colour in colourmap in "
+		    "MainWindow::draw_fax_from_socket_notifier()\n");
+      }
+      else {
+	GobjHandle<GdkGC> red_gc_h(gdk_gc_new(drawable_p));
+	gdk_gc_set_foreground(red_gc_h, &red);
 
-      GdkGC* white_gc_p = gtk_widget_get_style(drawing_area_p)->white_gc;
+	GdkGC* white_gc_p = gtk_widget_get_style(drawing_area_p)->white_gc;
 
-      gdk_draw_arc(drawable_p, white_gc_p,
-		   true, 6, 6,
-		   standard_size - 12, standard_size - 12,
-		   0, 64 * 360);
-			    
-      gdk_draw_arc(drawable_p, red_gc_h,
-		   true, 8, 8,
-		   standard_size - 16, standard_size - 16,
-		   0, 64 * 360);
+	gdk_draw_arc(drawable_p, white_gc_p,
+		     true, 6, 6,
+		     standard_size - 12, standard_size - 12,
+		     0, 64 * 360);
+	
+	gdk_draw_arc(drawable_p, red_gc_h,
+		     true, 8, 8,
+		     standard_size - 16, standard_size - 16,
+		     0, 64 * 360);
+      }
     }
 
     else {
@@ -1642,8 +1668,9 @@ bool MainWindow::read_error_pipe_slot(void) {
   ssize_t result;
 
   while ((result = MainWindow::error_pipe.read(pipe_buffer, PIPE_BUF)) > 0) {
-    pipe_buffer[result] = 0;
-    text_window.write_red_slot(pipe_buffer);
+    SharedHandle<char*> output_h(error_pipe_reassembler(pipe_buffer, result));
+    if (output_h.get()) text_window.write_red_slot(output_h.get());
+    else write_error(gettext("Invalid Utf8 received in MainWindow::read_error_pipe_slot()\n"));
   }
   return true; // retain connection for further reads
 }
@@ -1680,14 +1707,18 @@ MessageText::MessageText(void):
   red.red = static_cast<guint16>(0.8 * 65535.0);
   red.green = 0;
   red.blue = 0;
+  red_tag_h = GobjHandle<GtkTextTag>(gtk_text_tag_new(0));
 
   GdkColormap* colormap_p = gtk_widget_get_default_colormap();
-  gdk_colormap_alloc_color(colormap_p, &red, false, true);
-
-  red_tag_h = GobjHandle<GtkTextTag>(gtk_text_tag_new(0));
-  g_object_set(red_tag_h.get(),
-	       "foreground-gdk", &red,
-	       0);
+  if (!gdk_colormap_alloc_color(colormap_p, &red, false, true)) {
+    write_error("Error allocating colour in colourmap in "
+		"MessageText::MessageText()\n");
+  }
+  else {
+    g_object_set(red_tag_h.get(),
+		 "foreground-gdk", &red,
+		 static_cast<void*>(0));
+  }
 
   GtkTextBuffer* buffer_p = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view_p));
   gtk_text_tag_table_add(gtk_text_buffer_get_tag_table(buffer_p), red_tag_h);
@@ -1893,7 +1924,7 @@ void MessageText::cleanify(std::string& message) {
     else chunk_size = chunk_end - chunk_start;
     if (message.substr(chunk_start, chunk_size).find(" Copyright ") != std::string::npos
 	|| message.substr(chunk_start, chunk_size).find(" compiled ") != std::string::npos
-	|| message.substr(chunk_start, chunk_size).find("Error: terminating on signal 15") != std::string::npos) {
+	|| message.substr(chunk_start, chunk_size).find("terminating on signal 15") != std::string::npos) {
       message.erase(chunk_start, chunk_size);
       // if we have erased this chunk then we don't want to reset chunk_start
       // unless we have finished examining message
@@ -1956,7 +1987,7 @@ void MessageText::write_red_slot(const char* message) {
   GtkTextIter end_iter;
   gtk_text_buffer_get_end_iter(buffer_p, &end_iter);
   gtk_text_buffer_insert_with_tags(buffer_p, &end_iter, temp.data(), temp.size(),
-				   red_tag_h, 0);
+				   red_tag_h, static_cast<void*>(0));
 
   if (scrolling) {
     gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(text_view_p), end_mark_p,
@@ -2005,27 +2036,21 @@ StatusLine::StatusLine(const int size):
   gtk_widget_show_all(get_main_widget());
 }
 
-void StatusLine::set_status_line(GtkStyle* new_style_p, int new_height) {
+void StatusLine::set_status_line(int new_height) {
 
+  // set the height
   if (new_height < standard_size + 2) new_height = standard_size + 2;
   gtk_widget_set_size_request(get_main_widget(), -1, new_height);
-  // make the status label display in red
-  // Note: we need to take a copy of the existing style, as the existing
-  // style is shared by all the program's widgets.  If we operate
-  // directly on the GtkStyle object returned by gtk_widget_get_style()
-  // we will alter it for everything!
-  GobjHandle<GtkStyle> reset_style_h(gtk_style_copy(new_style_p));
 
+  // make the status label display in red
   GdkColor red;
   red.red = static_cast<guint16>(0.9 * 65535.0);
   red.green = 0;
   red.blue = 0;
 
-  GdkColormap* colormap_p = gtk_widget_get_default_colormap();
-  gdk_colormap_alloc_color(colormap_p, &red, false, true);
-  reset_style_h->fg[GTK_STATE_NORMAL] = red;
-
-  gtk_widget_set_style(status_label_p, reset_style_h);
+  // we don't need to allocate the GdkColor red object before it is
+  // used by gtk_widget_modify_fg() - that function will do it itself
+  gtk_widget_modify_fg(status_label_p, GTK_STATE_NORMAL, &red);
 }
 
 void StatusLine::write_status_slot(const char* text) {
@@ -2061,20 +2086,16 @@ ssize_t write_error(const char* message) {
   // will be lost unless the caller checks the return value and acts
   // accordingly
 
-  // if the static error pipe writes an error in its constructor then
-  // write_error_mutex_p is NULL, so check for this before attempting
-  // to lock the mutex
-  if (write_error_mutex_p) write_error_mutex_p->lock();
-
   ssize_t result = 0;
   ssize_t written = 0;
   ssize_t to_write_count = std::strlen(message);
   
   // if write_error_mutex_p is NULL then this must be the static
-  // pipe writing an error in its constructor so check for this
-  // (as well as whether MainWindow::connected_to_stderr is false)
-  // before writing to the pipe
+  // pipe being written to before the MainWindow constructor is
+  // entered - we might was well check this condition in this
+  // function although it should not happen
   if (!MainWindow::connected_to_stderr && write_error_mutex_p) {
+    Thread::Mutex::Lock lock(*write_error_mutex_p);
     do {
       result = MainWindow::error_pipe.write(message + written, to_write_count);
       if (result > 0) {
@@ -2085,6 +2106,12 @@ ssize_t write_error(const char* message) {
   }
 
   else {
+    // no mutex is required in this case: MainWindow::connect_to_stderr()
+    // is only called after fork()ing and it is for the new process to
+    // synchronise its writes to standard error (in efax-gtk child
+    // processes are all single-threaded anyway) and as between different
+    // processes, POSIX guarantees that writes not larger than PIPE_BUF
+    // in size will not be interleaved
     do {
       result = ::write(2, message + written, to_write_count);
       if (result > 0) {
@@ -2093,7 +2120,6 @@ ssize_t write_error(const char* message) {
       }
     } while (to_write_count && (result != -1 || errno == EINTR));
   }
-  if (write_error_mutex_p) write_error_mutex_p->unlock();
   return written;
 }
 
