@@ -10,16 +10,10 @@ COPYING distributed with the source files.
 #define PROG_DEFS_H
 
 #include <unistd.h>
-#include "config.h"
+#include <config.h>
 
 // deal with any configuration issues arising from config.h
 
-#ifndef HAVE_BOOL
-typedef int bool;
-const bool false = 0;
-const bool true = 1;
-#endif
-
 #ifndef ENABLE_NLS
 inline const char* gettext(const char* text) {
   return text;
@@ -50,6 +44,7 @@ typedef unsigned int socklen_t;
 #define ARG_ERROR 16
 #define EXEC_ERROR 17
 #define FORK_ERROR 18
+#define NOTIFIER_ERROR 19
 
 #define RC_FILE "efax-gtkrc"
 
@@ -57,7 +52,7 @@ typedef unsigned int socklen_t;
 #include <vector>
 #include <string>
 #include <gdk/gdkpixbuf.h>
-#include "gobj_handle.h"
+#include "utils/gobj_handle.h"
 
 // forward declaration for Prog_config
 namespace Thread {
@@ -67,7 +62,6 @@ namespace Thread {
 
 using namespace std;    // until I get a Std C++ compliant compiler to test with
 
-#define MAX_TEMP_NAME 12
 #define PIPE_NAME "/.efax-gtk-pipe"
 
 /************************* threading issues *************************
@@ -205,7 +199,6 @@ struct Prog_config {
   std::string homedir;
   std::string working_dir;
   char rings;
-  char receive_dirname[MAX_TEMP_NAME + 1];
   bool tone_dial;
   bool found_rcfile;
   bool GPL_flag;
@@ -215,6 +208,7 @@ struct Prog_config {
   bool other_sock_client_address;
   bool fax_received_popup;
   bool fax_received_exec;
+  bool addr_in_header;
   std::string my_name;
   std::string my_number;
   std::string page_size;  // either 'a4' 'letter' or 'legal'
