@@ -17,6 +17,7 @@ COPYING distributed with the source files.
 
 #include <gtk/gtkcontainer.h>
 #include <gtk/gtkalignment.h>
+#include <gtk/gtkmisc.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtklabel.h>
@@ -41,12 +42,16 @@ COPYING distributed with the source files.
 #include "settings.h"
 #include "settings_icons.h"
 #include "dialogs.h"
-#include "gobj_handle.h"
+#include "utils/gobj_handle.h"
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
 #endif
 
+static inline bool not_ascii_char(char c) {
+  if (static_cast<signed char>(c) < 0) return true;
+  return false;
+}
 
 namespace { // callback for internal use only
 
@@ -76,19 +81,14 @@ IdentityPage::IdentityPage(const int standard_size):
   GtkTable* table_p = GTK_TABLE(gtk_table_new(2, 3, false));
   gtk_container_add(GTK_CONTAINER(get_main_widget()), GTK_WIDGET(table_p));
 
-  GtkWidget* name_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* number_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-
   GtkWidget* name_label_p = gtk_label_new(gettext("Name: "));
   GtkWidget* number_label_p = gtk_label_new(gettext("Number: "));
 
   gtk_label_set_justify(GTK_LABEL(name_label_p), GTK_JUSTIFY_RIGHT);
   gtk_label_set_justify(GTK_LABEL(number_label_p), GTK_JUSTIFY_RIGHT);
 
-  gtk_container_add(GTK_CONTAINER(name_label_alignment_p),
-		    name_label_p);
-  gtk_container_add(GTK_CONTAINER(number_label_alignment_p),
-		    number_label_p);
+  gtk_misc_set_alignment(GTK_MISC(name_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(number_label_p), 1, 0.5);
 
   name_entry_p = gtk_entry_new();
   number_entry_p = gtk_entry_new();
@@ -106,7 +106,7 @@ IdentityPage::IdentityPage(const int standard_size):
     gtk_container_add(GTK_CONTAINER(number_help_button_p), image_p);
   }
 
-  gtk_table_attach(table_p, name_label_alignment_p,
+  gtk_table_attach(table_p, name_label_p,
 		   0, 1, 0, 1,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/3, standard_size);
@@ -119,7 +119,7 @@ IdentityPage::IdentityPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/3, standard_size);
 
-  gtk_table_attach(table_p, number_label_alignment_p,
+  gtk_table_attach(table_p, number_label_p,
 		   0, 1, 1, 2,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/3, standard_size);
@@ -207,13 +207,6 @@ ModemPage::ModemPage(const int standard_size):
   GtkTable* table_p = GTK_TABLE(gtk_table_new(6, 3, false));
   gtk_container_add(GTK_CONTAINER(get_main_widget()), GTK_WIDGET(table_p));
 
-  GtkWidget* device_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* lock_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* capabilities_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* rings_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* class_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* dialmode_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-
   GtkWidget* device_label_p = gtk_label_new(gettext("Serial Device: "));
   GtkWidget* lock_label_p = gtk_label_new(gettext("Lock File: "));
   GtkWidget* capabilities_label_p = gtk_label_new(gettext("Capabilities: "));
@@ -228,18 +221,12 @@ ModemPage::ModemPage(const int standard_size):
   gtk_label_set_justify(GTK_LABEL(class_label_p), GTK_JUSTIFY_RIGHT);
   gtk_label_set_justify(GTK_LABEL(dialmode_label_p), GTK_JUSTIFY_RIGHT);
 
-  gtk_container_add(GTK_CONTAINER(device_label_alignment_p),
-		    device_label_p);
-  gtk_container_add(GTK_CONTAINER(lock_label_alignment_p),
-		    lock_label_p);
-  gtk_container_add(GTK_CONTAINER(capabilities_label_alignment_p),
-		    capabilities_label_p);
-  gtk_container_add(GTK_CONTAINER(rings_label_alignment_p),
-		    rings_label_p);
-  gtk_container_add(GTK_CONTAINER(class_label_alignment_p),
-		    class_label_p);
-  gtk_container_add(GTK_CONTAINER(dialmode_label_alignment_p),
-		    dialmode_label_p);
+  gtk_misc_set_alignment(GTK_MISC(device_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(lock_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(capabilities_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(rings_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(class_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(dialmode_label_p), 1, 0.5);
 
   auto_button_p = gtk_radio_button_new_with_label(0, gettext("Auto"));
   class1_button_p = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(auto_button_p),
@@ -309,7 +296,7 @@ ModemPage::ModemPage(const int standard_size):
     gtk_container_add(GTK_CONTAINER(dialmode_help_button_p), image_p);
   }
 
-  gtk_table_attach(table_p, device_label_alignment_p,
+  gtk_table_attach(table_p, device_label_p,
 		   0, 1, 0, 1,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/5, standard_size/5);
@@ -322,7 +309,7 @@ ModemPage::ModemPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/3, standard_size/5);
 
-  gtk_table_attach(table_p, lock_label_alignment_p,
+  gtk_table_attach(table_p, lock_label_p,
 		   0, 1, 1, 2,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/5, standard_size/5);
@@ -335,7 +322,7 @@ ModemPage::ModemPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/3, standard_size/5);
 
-  gtk_table_attach(table_p, capabilities_label_alignment_p,
+  gtk_table_attach(table_p, capabilities_label_p,
 		   0, 1, 2, 3,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/5, standard_size/5);
@@ -348,7 +335,7 @@ ModemPage::ModemPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/3, standard_size/5);
 
-  gtk_table_attach(table_p, rings_label_alignment_p,
+  gtk_table_attach(table_p, rings_label_p,
 		   0, 1, 3, 4,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/5, standard_size/5);
@@ -361,7 +348,7 @@ ModemPage::ModemPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/3, standard_size/5);
 
-  gtk_table_attach(table_p, class_label_alignment_p,
+  gtk_table_attach(table_p, class_label_p,
 		   0, 1, 4, 5,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/5, standard_size/5);
@@ -374,7 +361,7 @@ ModemPage::ModemPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/3, standard_size/5);
 
-  gtk_table_attach(table_p, dialmode_label_alignment_p,
+  gtk_table_attach(table_p, dialmode_label_p,
 		   0, 1, 5, 6,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/5, standard_size/5);
@@ -529,10 +516,6 @@ ParmsPage::ParmsPage(const int standard_size):
   GtkTable* table_p = GTK_TABLE(gtk_table_new(3, 3, false));
   gtk_container_add(GTK_CONTAINER(get_main_widget()), GTK_WIDGET(table_p));
 
-  GtkWidget* init_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* reset_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* parms_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-
   GtkWidget* init_label_p = gtk_label_new(gettext("Initialization Params:"));
   GtkWidget* reset_label_p = gtk_label_new(gettext("Reset Params: "));
   GtkWidget* parms_label_p = gtk_label_new(gettext("Other Params: "));
@@ -541,12 +524,9 @@ ParmsPage::ParmsPage(const int standard_size):
   gtk_label_set_justify(GTK_LABEL(reset_label_p), GTK_JUSTIFY_RIGHT);
   gtk_label_set_justify(GTK_LABEL(parms_label_p), GTK_JUSTIFY_RIGHT);
 
-  gtk_container_add(GTK_CONTAINER(init_label_alignment_p),
-		    init_label_p);
-  gtk_container_add(GTK_CONTAINER(reset_label_alignment_p),
-		    reset_label_p);
-  gtk_container_add(GTK_CONTAINER(parms_label_alignment_p),
-		    parms_label_p);
+  gtk_misc_set_alignment(GTK_MISC(init_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(reset_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(parms_label_p), 1, 0.5);
 
   init_entry_p = gtk_entry_new();
   reset_entry_p = gtk_entry_new();
@@ -569,7 +549,7 @@ ParmsPage::ParmsPage(const int standard_size):
     gtk_container_add(GTK_CONTAINER(parms_help_button_p), image_p);
   }
 
-  gtk_table_attach(table_p, init_label_alignment_p,
+  gtk_table_attach(table_p, init_label_p,
 		   0, 1, 0, 1,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/3, standard_size/2);
@@ -582,7 +562,7 @@ ParmsPage::ParmsPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/3, standard_size/2);
 
-  gtk_table_attach(table_p, reset_label_alignment_p,
+  gtk_table_attach(table_p, reset_label_p,
 		   0, 1, 1, 2,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/3, standard_size/2);
@@ -595,7 +575,7 @@ ParmsPage::ParmsPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/3, standard_size/2);
 
-  gtk_table_attach(table_p, parms_label_alignment_p,
+  gtk_table_attach(table_p, parms_label_p,
 		   0, 1, 2, 3,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/3, standard_size/2);
@@ -686,10 +666,6 @@ PrintPage::PrintPage(const int standard_size):
   GtkTable* table_p = GTK_TABLE(gtk_table_new(3, 3, false));
   gtk_container_add(GTK_CONTAINER(get_main_widget()), GTK_WIDGET(table_p));
   
-  GtkWidget* command_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* shrink_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* popup_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-
   GtkWidget* command_label_p = gtk_label_new(gettext("Print Program: "));
   GtkWidget* shrink_label_p = gtk_label_new(gettext("Print Shrink (50-100):"));
   GtkWidget* popup_label_p = gtk_label_new(gettext("Popup confirmation dialog\nbefore printing"));
@@ -697,14 +673,11 @@ PrintPage::PrintPage(const int standard_size):
   gtk_label_set_justify(GTK_LABEL(command_label_p), GTK_JUSTIFY_RIGHT);
   gtk_label_set_justify(GTK_LABEL(shrink_label_p), GTK_JUSTIFY_RIGHT);
   gtk_label_set_justify(GTK_LABEL(popup_label_p), GTK_JUSTIFY_RIGHT);
-  
-  gtk_container_add(GTK_CONTAINER(command_label_alignment_p),
-		    command_label_p);
-  gtk_container_add(GTK_CONTAINER(shrink_label_alignment_p),
-		    shrink_label_p);
-  gtk_container_add(GTK_CONTAINER(popup_label_alignment_p),
-		    popup_label_p);
 
+  gtk_misc_set_alignment(GTK_MISC(command_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(shrink_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(popup_label_p), 1, 0.5);
+  
   command_entry_p = gtk_entry_new();
   shrink_entry_p = gtk_entry_new();
   gtk_widget_set_size_request(command_entry_p, standard_size * 7, standard_size);
@@ -730,7 +703,7 @@ PrintPage::PrintPage(const int standard_size):
     gtk_container_add(GTK_CONTAINER(popup_help_button_p), image_p);
   }
 
-  gtk_table_attach(table_p, command_label_alignment_p,
+  gtk_table_attach(table_p, command_label_p,
 		   0, 1, 0, 1,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/3, standard_size/2);
@@ -743,7 +716,7 @@ PrintPage::PrintPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/3, standard_size/2);
 
-  gtk_table_attach(table_p, shrink_label_alignment_p,
+  gtk_table_attach(table_p, shrink_label_p,
 		   0, 1, 1, 2,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/3, standard_size/2);
@@ -756,7 +729,7 @@ PrintPage::PrintPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/3, standard_size/2);
 
-  gtk_table_attach(table_p, popup_label_alignment_p,
+  gtk_table_attach(table_p, popup_label_p,
 		   0, 1, 2, 3,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/4, standard_size/2);
@@ -842,8 +815,8 @@ ViewPage::ViewPage(const int standard_size):
   GtkTable* table_p = GTK_TABLE(gtk_table_new(1, 3, false));
   gtk_container_add(GTK_CONTAINER(get_main_widget()), GTK_WIDGET(table_p));
    
-  GtkWidget* ps_view_command_label_p = gtk_label_new(gettext("Postscript Viewer\nProgram:"));
-  gtk_label_set_justify(GTK_LABEL(ps_view_command_label_p), GTK_JUSTIFY_LEFT);
+  GtkWidget* ps_view_command_label_p = gtk_label_new(gettext("Postscript viewer\nprogram:"));
+  gtk_label_set_justify(GTK_LABEL(ps_view_command_label_p), GTK_JUSTIFY_RIGHT);
 
   ps_view_command_entry_p = gtk_entry_new();
   gtk_widget_set_size_request(ps_view_command_entry_p, standard_size * 7, standard_size);
@@ -928,11 +901,6 @@ SockPage::SockPage(const int standard_size):
   GtkTable* table_p = GTK_TABLE(gtk_table_new(4, 3, false));
   gtk_container_add(GTK_CONTAINER(get_main_widget()), GTK_WIDGET(table_p));
 
-  GtkWidget* run_server_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* popup_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* port_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* client_address_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-
   GtkWidget* run_server_label_p = gtk_label_new(gettext("Run socket server"));
   GtkWidget* popup_label_p = gtk_label_new(gettext("Popup dialog when fax\nreceived from socket"));
   GtkWidget* port_label_p = gtk_label_new(gettext("Port to which faxes to\nbe sent:"));
@@ -943,14 +911,10 @@ SockPage::SockPage(const int standard_size):
   gtk_label_set_justify(GTK_LABEL(port_label_p), GTK_JUSTIFY_RIGHT);
   gtk_label_set_justify(GTK_LABEL(client_address_label_p), GTK_JUSTIFY_RIGHT);
 
-  gtk_container_add(GTK_CONTAINER(run_server_label_alignment_p),
-		    run_server_label_p);
-  gtk_container_add(GTK_CONTAINER(popup_label_alignment_p),
-		    popup_label_p);
-  gtk_container_add(GTK_CONTAINER(port_label_alignment_p),
-		    port_label_p);
-  gtk_container_add(GTK_CONTAINER(client_address_label_alignment_p),
-		    client_address_label_p);
+  gtk_misc_set_alignment(GTK_MISC(run_server_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(popup_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(port_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(client_address_label_p), 1, 0.5);
 
   GtkWidget* run_server_button_alignment_p = gtk_alignment_new(0, 0.5, 0, 1);
   GtkWidget* popup_button_alignment_p = gtk_alignment_new(0, 0.5, 0, 1);
@@ -1011,7 +975,7 @@ SockPage::SockPage(const int standard_size):
     gtk_container_add(GTK_CONTAINER(client_address_help_button_p), image_p);
   }
 
-  gtk_table_attach(table_p, run_server_label_alignment_p,
+  gtk_table_attach(table_p, run_server_label_p,
 		   0, 1, 0, 1,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/4, standard_size/3);
@@ -1024,7 +988,7 @@ SockPage::SockPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/4, standard_size/3);
 
-  gtk_table_attach(table_p, popup_label_alignment_p,
+  gtk_table_attach(table_p, popup_label_p,
 		   0, 1, 1, 2,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/4, standard_size/3);
@@ -1037,7 +1001,7 @@ SockPage::SockPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/4, standard_size/3);
 
-  gtk_table_attach(table_p, port_label_alignment_p,
+  gtk_table_attach(table_p, port_label_p,
 		   0, 1, 2, 3,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/4, standard_size/3);
@@ -1050,7 +1014,7 @@ SockPage::SockPage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/4, standard_size/3);
 
-  gtk_table_attach(table_p, client_address_label_alignment_p,
+  gtk_table_attach(table_p, client_address_label_p,
 		   0, 1, 3, 4,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/4, standard_size/3);
@@ -1206,23 +1170,20 @@ ReceivePage::ReceivePage(const int standard_size):
   GtkTable* table_p = GTK_TABLE(gtk_table_new(2, 3, false));
   gtk_container_add(GTK_CONTAINER(get_main_widget()), GTK_WIDGET(table_p));
 
-  GtkWidget* popup_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* exec_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* popup_button_alignment_p = gtk_alignment_new(0, 0.5, 0, 1);
-  GtkWidget* exec_button_alignment_p = gtk_alignment_new(0, 0.5, 0, 1);
-  
   GtkWidget* popup_label_p = gtk_label_new(gettext("Popup dialog when fax\nreceived by modem"));
   GtkWidget* exec_label_p = gtk_label_new(gettext("Execute a program or script\nwhen fax received by modem"));
-  popup_button_p = gtk_check_button_new();
-  exec_button_p = gtk_check_button_new();
 
   gtk_label_set_justify(GTK_LABEL(popup_label_p), GTK_JUSTIFY_RIGHT);
   gtk_label_set_justify(GTK_LABEL(exec_label_p), GTK_JUSTIFY_RIGHT);
 
-  gtk_container_add(GTK_CONTAINER(popup_label_alignment_p),
-		    popup_label_p);
-  gtk_container_add(GTK_CONTAINER(exec_label_alignment_p),
-		    exec_label_p);
+  gtk_misc_set_alignment(GTK_MISC(popup_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(exec_label_p), 1, 0.5);
+
+  GtkWidget* popup_button_alignment_p = gtk_alignment_new(0, 0.5, 0, 1);
+  GtkWidget* exec_button_alignment_p = gtk_alignment_new(0, 0.5, 0, 1);
+  popup_button_p = gtk_check_button_new();
+  exec_button_p = gtk_check_button_new();
+  
   gtk_container_add(GTK_CONTAINER(popup_button_alignment_p),
 		    popup_button_p);
   gtk_container_add(GTK_CONTAINER(exec_button_alignment_p),
@@ -1255,7 +1216,7 @@ ReceivePage::ReceivePage(const int standard_size):
     gtk_container_add(GTK_CONTAINER(exec_help_button_p), image_p);
   }
 
-  gtk_table_attach(table_p, popup_label_alignment_p,
+  gtk_table_attach(table_p, popup_label_p,
 		   0, 1, 0, 1,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/3, standard_size);
@@ -1268,7 +1229,7 @@ ReceivePage::ReceivePage(const int standard_size):
 		   GTK_SHRINK, GTK_SHRINK,
 		   standard_size/3, standard_size);
 
-  gtk_table_attach(table_p, exec_label_alignment_p,
+  gtk_table_attach(table_p, exec_label_p,
 		   0, 1, 1, 2,
 		   GTK_FILL, GTK_SHRINK,
 		   standard_size/3, standard_size);
@@ -1435,6 +1396,9 @@ void PagePageCB::page_page_button_clicked(GtkWidget* widget_p, void* data) {
   else if (widget_p == instance_p->res_help_button_p) {
     message_index = PageMessages::res;
   }
+  else if (widget_p == instance_p->header_help_button_p) {
+    message_index = PageMessages::header;
+  }
   else {
     write_error("Callback error in PagePageCB::page_page_button_clicked()\n");
     return;
@@ -1448,23 +1412,21 @@ void PagePageCB::page_page_button_clicked(GtkWidget* widget_p, void* data) {
 PagePage::PagePage(const int standard_size):
                        MainWidgetBase(gtk_alignment_new(0.5, 0.5, 1, 0)) {
 
-  GtkTable* table_p = GTK_TABLE(gtk_table_new(2, 3, false));
+  GtkTable* table_p = GTK_TABLE(gtk_table_new(3, 3, false));
   gtk_container_add(GTK_CONTAINER(get_main_widget()), GTK_WIDGET(table_p));
 
-  GtkWidget* page_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-  GtkWidget* res_label_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
-
   GtkWidget* page_label_p = gtk_label_new(gettext("Page Size: "));
   GtkWidget* res_label_p = gtk_label_new(gettext("Sent Fax Resolution: "));
+  GtkWidget* header_label_p = gtk_label_new(gettext("Include destination\nfax number on fax\npage top header line"));
 
   gtk_label_set_justify(GTK_LABEL(page_label_p), GTK_JUSTIFY_RIGHT);
   gtk_label_set_justify(GTK_LABEL(res_label_p), GTK_JUSTIFY_RIGHT);
+  gtk_label_set_justify(GTK_LABEL(header_label_p), GTK_JUSTIFY_RIGHT);
+
+  gtk_misc_set_alignment(GTK_MISC(page_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(res_label_p), 1, 0.5);
+  gtk_misc_set_alignment(GTK_MISC(header_label_p), 1, 0.5);
 
-  gtk_container_add(GTK_CONTAINER(page_label_alignment_p),
-		    page_label_p);
-  gtk_container_add(GTK_CONTAINER(res_label_alignment_p),
-		    res_label_p);
-  
   a4_button_p = gtk_radio_button_new_with_label(0, "A4");
   letter_button_p = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(a4_button_p),
 								"Letter");
@@ -1473,9 +1435,11 @@ PagePage::PagePage(const int standard_size):
   standard_button_p = gtk_radio_button_new_with_label(0, gettext("Standard"));
   fine_button_p = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(standard_button_p),
 							      gettext("Fine"));
+  header_check_button_p = gtk_check_button_new();
 
   GtkWidget* page_box_alignment_p = gtk_alignment_new(0, 0.5, 0, 1);
   GtkWidget* res_box_alignment_p = gtk_alignment_new(0, 0.5, 0, 1);
+  GtkWidget* header_button_alignment_p = gtk_alignment_new(1, 0.5, 0, 1);
 
   GtkBox* page_box_p = GTK_BOX(gtk_hbox_new(false, 2));
   GtkBox* res_box_p = GTK_BOX(gtk_hbox_new(false, 2));
@@ -1497,12 +1461,17 @@ PagePage::PagePage(const int standard_size):
   gtk_container_add(GTK_CONTAINER(res_box_alignment_p), GTK_WIDGET(res_box_p));
   GtkWidget* res_frame_p = gtk_frame_new(0);
   gtk_container_add(GTK_CONTAINER(res_frame_p), res_box_alignment_p);
+  gtk_container_add(GTK_CONTAINER(header_button_alignment_p),
+		    header_check_button_p);
 
+  // set the defaults
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(a4_button_p), true);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fine_button_p), true);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(header_check_button_p), true);
 
   page_help_button_p = gtk_button_new();
   res_help_button_p = gtk_button_new();
+  header_help_button_p = gtk_button_new();
   GtkWidget* image_p;
   { // scope block for the GobjHandle
     GobjHandle<GdkPixbuf> pixbuf_h(gdk_pixbuf_new_from_xpm_data(help_xpm));
@@ -1510,44 +1479,63 @@ PagePage::PagePage(const int standard_size):
     gtk_container_add(GTK_CONTAINER(page_help_button_p), image_p);
     image_p = gtk_image_new_from_pixbuf(pixbuf_h);
     gtk_container_add(GTK_CONTAINER(res_help_button_p), image_p);
+    image_p = gtk_image_new_from_pixbuf(pixbuf_h);
+    gtk_container_add(GTK_CONTAINER(header_help_button_p), image_p);
   }
 
-  gtk_table_attach(table_p, page_label_alignment_p,
+  gtk_table_attach(table_p, page_label_p,
 		   0, 1, 0, 1,
 		   GTK_FILL, GTK_SHRINK,
-		   standard_size/3, standard_size);
+		   standard_size/3, standard_size/2);
   gtk_table_attach(table_p, page_frame_p,
 		   1, 2, 0, 1,
 		   GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_SHRINK,
-		   standard_size/3, standard_size);
+		   standard_size/3, standard_size/2);
   gtk_table_attach(table_p, page_help_button_p,
 		   2, 3, 0, 1,
 		   GTK_SHRINK, GTK_SHRINK,
-		   standard_size/3, standard_size);
+		   standard_size/3, standard_size/2);
   
-  gtk_table_attach(table_p, res_label_alignment_p,
+  gtk_table_attach(table_p, res_label_p,
 		   0, 1, 1, 2,
 		   GTK_FILL, GTK_SHRINK,
-		   standard_size/3, standard_size);
+		   standard_size/3, standard_size/2);
   gtk_table_attach(table_p, res_frame_p,
 		   1, 2, 1, 2,
 		   GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_SHRINK,
-		   standard_size/3, standard_size);
+		   standard_size/3, standard_size/2);
   gtk_table_attach(table_p, res_help_button_p,
 		   2, 3, 1, 2,
 		   GTK_SHRINK, GTK_SHRINK,
-		   standard_size/3, standard_size);
+		   standard_size/3, standard_size/2);
   
+  gtk_table_attach(table_p, header_label_p,
+		   0, 1, 2, 3,
+		   GTK_FILL, GTK_SHRINK,
+		   standard_size/4, standard_size/3);
+  gtk_table_attach(table_p, header_button_alignment_p,
+		   1, 2, 2, 3,
+		   GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_SHRINK,
+		   standard_size/4, standard_size/3);
+  gtk_table_attach(table_p, header_help_button_p,
+		   2, 3, 2, 3,
+		   GTK_SHRINK, GTK_SHRINK,
+		   standard_size/4, standard_size/3);
+
   g_signal_connect(G_OBJECT(page_help_button_p), "clicked",
 		   G_CALLBACK(PagePageCB::page_page_button_clicked), this);
   g_signal_connect(G_OBJECT(res_help_button_p), "clicked",
 		   G_CALLBACK(PagePageCB::page_page_button_clicked), this);
+  g_signal_connect(G_OBJECT(header_help_button_p), "clicked",
+		   G_CALLBACK(PagePageCB::page_page_button_clicked), this);
 
   GtkTooltips* tooltips_p = gtk_tooltips_new();
   gtk_tooltips_set_tip(tooltips_p, page_help_button_p,
 		       help_messages.get_message(PageMessages::page).c_str(), "");
   gtk_tooltips_set_tip(tooltips_p, res_help_button_p,
 		       help_messages.get_message(PageMessages::res).c_str(), "");
+  gtk_tooltips_set_tip(tooltips_p, header_help_button_p,
+		       help_messages.get_message(PageMessages::header).c_str(), "");
 
   gtk_container_set_border_width(GTK_CONTAINER(get_main_widget()), standard_size/3);
 }
@@ -1567,6 +1555,14 @@ std::string PagePage::get_res(void) const {
   return return_val;
 }
 
+std::string PagePage::get_addr_in_header(void) const {
+
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(header_check_button_p))) {
+    return std::string("Yes");
+  }
+  return std::string("No");
+}
+
 void PagePage::set_page(const std::string& page_string) {
 
   std::string temp(Utf8::to_lower(page_string));
@@ -1592,6 +1588,15 @@ void PagePage::set_res(const std::string& res_string) {
   }
 }
 
+void PagePage::set_addr_in_header(const std::string& header_string) {
+
+  std::string temp(Utf8::to_lower(header_string));
+  if (!temp.compare("no")) {
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(header_check_button_p), false);
+  }
+  else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(header_check_button_p), true);
+}
+
 void PagePage::clear(void) {
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(a4_button_p), true);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fine_button_p), true);
@@ -1775,7 +1780,17 @@ bool SettingsDialog::write_config(void) {
   std::string name_line("NAME: ");
   temp = identity_page.get_name();
   strip(temp);
-  name_line += temp;
+  if (!is_ascii(temp)) {
+    return_val = false;
+    InfoDialog dialog(gettext("Invalid user name specified in \"Identity\" tab - "
+			      "it must be in plain ASCII characters.  If this "
+			      "is a problem, leave it blank as the fax station "
+			      "number will always be given on the top header"),
+		      gettext("Config Error"),
+		      GTK_MESSAGE_ERROR, get_win());
+    dialog.exec();
+  }
+  else name_line += temp;
   
   std::string number_line("NUMBER: ");
   temp =  identity_page.get_number();
@@ -2060,6 +2075,15 @@ bool SettingsDialog::write_config(void) {
     fax_received_prog_line += temp;
   }
   
+  std::string addr_in_header_line;
+  temp = page_page.get_addr_in_header();
+  strip(temp);
+  if (temp.empty()) addr_in_header_line = "#ADDR_IN_HEADER: Yes";
+  else {
+    addr_in_header_line = "ADDR_IN_HEADER: ";
+    addr_in_header_line += temp;
+  }
+
   std::string logfile_line;
   temp = logging_page.get_logfile();
   strip(temp);
@@ -2096,6 +2120,7 @@ bool SettingsDialog::write_config(void) {
     bool found_fax_received_popup = false;
     bool found_fax_received_exec = false;
     bool found_fax_received_prog = false;
+    bool found_addr_in_header = false;
     bool found_log_file = false;
     
     const char terminating_line[] = "## end of " RC_FILE " ##";
@@ -2303,6 +2328,14 @@ bool SettingsDialog::write_config(void) {
 	  }
 	}
       
+	// look for "ADDR_IN_HEADER:"
+	else if (find_prog_parm("ADDR_IN_HEADER:", file_read) || find_prog_parm("#ADDR_IN_HEADER:", file_read)) {
+	  if (!found_addr_in_header) {
+	    found_addr_in_header = true;
+	    file_list.push_back(addr_in_header_line);
+	  }
+	}
+      
 	// look for "LOG_FILE:"
 	else if (find_prog_parm("LOG_FILE:", file_read) || find_prog_parm("#LOG_FILE:", file_read)) {
 	  if (!found_log_file) {
@@ -2340,6 +2373,7 @@ bool SettingsDialog::write_config(void) {
     if (found_sock_other_addresses == false) file_list.push_back(sock_other_addresses_line);
     if (found_fax_received_popup == false) file_list.push_back(fax_received_popup_line);
     if (found_fax_received_exec == false) file_list.push_back(fax_received_exec_line);
+    if (found_addr_in_header == false) file_list.push_back(addr_in_header_line);
 
     try {
       if (found_name == false) file_list.push_back(Utf8::locale_from_utf8(name_line));
@@ -2451,7 +2485,7 @@ void SettingsDialog::read_config(bool search_localfile) {
 	
 	  // look for "NAME:"
 	  if (get_prog_parm("NAME:", file_read, result)) {
-	    identity_page.set_name(result);
+	    if (is_ascii(result)) identity_page.set_name(result);
 	  }
 	
 	  // look for "NUMBER:"
@@ -2471,7 +2505,12 @@ void SettingsDialog::read_config(bool search_localfile) {
 
 	  // look for "CLASS:"
 	  else if (get_prog_parm("CLASS:", file_read, result)) {
-	    modem_page.set_class(result);
+	    if (!result.compare("1")
+		|| !result.compare("2")
+		|| !result.compare("2.0")) {
+	      modem_page.set_class(result);
+	    }
+	    else modem_page.set_class("2");
 	  }
 
 	  // look for "PAGE:"
@@ -2574,6 +2613,11 @@ void SettingsDialog::read_config(bool search_localfile) {
 	    receive_page.set_prog(result);
 	  }
 
+	  // look for "ADDR_IN_HEADER:"
+	  else if (get_prog_parm("ADDR_IN_HEADER:", file_read, result)) {
+	    page_page.set_addr_in_header(result);
+	  }
+
 	  // look for "LOG_FILE:"
 	  else if (get_prog_parm("LOG_FILE:", file_read, result, Utf8::filename_to_utf8)) {
 	    logging_page.set_logfile(result);
@@ -2711,3 +2755,12 @@ void SettingsDialog::strip(std::string& setting) {
     setting.erase(0, 1);
   }
 }
+
+bool SettingsDialog::is_ascii(const std::string& text) {
+  
+  std::string::const_iterator end_iter = text.end();
+  if (std::find_if(text.begin(), end_iter, not_ascii_char) == end_iter) {
+    return true;
+  }
+  return false;
+}
