@@ -20,11 +20,10 @@ COPYING distributed with the source files.
 
 #include <sigc++/sigc++.h>
 
-#include "window.h"
-#include "widget.h"
-
 #include "settings_help.h"
-#include "utf8_utils.h"
+#include "utils/window.h"
+#include "utils/widget.h"
+#include "utils/utf8_utils.h"
 
 
 namespace { // we put the function in anonymous namespace in settings.cpp
@@ -290,16 +289,20 @@ class PagePage: public MainWidgetBase {
   GtkWidget* legal_button_p;
   GtkWidget* standard_button_p;
   GtkWidget* fine_button_p;
+  GtkWidget* header_check_button_p;
   GtkWidget* page_help_button_p;
   GtkWidget* res_help_button_p;
+  GtkWidget* header_help_button_p;
 public:
   friend void PagePageCB::page_page_button_clicked(GtkWidget*, void*);
 
   sigc::signal2<void, const std::string&, const std::string&> show_help_sig;
   std::string get_page(void) const;
   std::string get_res(void) const;
+  std::string get_addr_in_header(void) const;
   void set_page(const std::string& page);
   void set_res(const std::string& res);
+  void set_addr_in_header(const std::string& res);
   void clear(void);
   PagePage(const int);
 };
@@ -342,6 +345,7 @@ class SettingsDialog: public sigc::trackable, public WinBase {
   void get_reset_settings(void);
   void show_help(const std::string& message, const std::string& caption);
   void strip(std::string&);
+  bool is_ascii(const std::string&);
 public:
   friend void SettingsDialogCB::settings_dialog_button_clicked(GtkWidget*, void*);
 
