@@ -41,7 +41,10 @@ std::string SettingsMessagesBase::get_message(std::vector<std::string>::size_typ
 
 IdentityMessages::IdentityMessages(void): SettingsMessagesBase(number + 1) {
 
-  messages[name] = gettext("Put the name to appear on the fax top header on sent faxes here");
+  messages[name] = gettext("Put the name to appear on the fax top header on sent faxes "
+			   "here.  It must be in plain ASCII characters.  If this is a "
+			   "problem, leave it blank as the fax station number will always "
+			   "be given on the top header");
   captions[name] = gettext("efax-gtk help: Name");
 
   messages[number] = gettext("Put the number to appear on the fax top header on sent faxes here.  "
@@ -204,7 +207,7 @@ LoggingMessages::LoggingMessages(void): SettingsMessagesBase(logfile + 1) {
   captions[logfile] = gettext("efax-gtk help: Log File");
 }
 
-PageMessages::PageMessages(void): SettingsMessagesBase(res + 1) {
+PageMessages::PageMessages(void): SettingsMessagesBase(header + 1) {
 
   messages[page] = gettext("Specify the page size for faxes here");
   captions[page] = gettext("efax-gtk help: Page Size");
@@ -212,6 +215,12 @@ PageMessages::PageMessages(void): SettingsMessagesBase(res + 1) {
   messages[res] = gettext("This specifies the resolution to which faxes are sent.  \"Standard\" "
 			  "gives 204x98 and \"Fine\" gives 204x196");
   captions[res] = gettext("efax-gtk help: Sent Fax Resolution");
+
+  messages[header] = gettext("This specifies whether the destination fax number is included in the "
+			     "top fax header line for sent faxes.  Uncheck the box if you do not "
+			     "want this to be shown (say because it includes a pay card access "
+			     "number)");
+  captions[header] = gettext("efax-gtk help: Fax top header line");
 }
 
 
