@@ -17,7 +17,7 @@ COPYING distributed with the source files.
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkwindow.h>
 
-#include "window.h"
+#include "utils/window.h"
 
 
 struct SettingsMessagesBase {
@@ -105,7 +105,7 @@ public:
 
 class PageMessages: private SettingsMessagesBase {
 public:
-  enum {page = 0, res};
+  enum {page = 0, res, header};
   PageMessages(void);
 
   using SettingsMessagesBase::get_caption;
