@@ -54,19 +54,20 @@ namespace sigc {
      are reproduced 'in extenso' 
   */
 
-  //sigc::ptrfun() --> SigC::slot (for function taking no arguments)
-  template <class R>
-    SigC::Slot0<R> 
-    ptr_fun(R (*func)())
-    { return func; }
-
-  // this is necessary for the next two template functions
+  // this is necessary for the next three template functions
   // sigc::slot<R> --> SigC::Slot0<R> (user declared slot taking member function with no argument)
   template <class R>
     struct slot : public SigC::Slot0<R> {
     slot(SigC::ObjectSlotNode* s): SigC::Slot0<R>(s) {}
+    slot(R (*f)()): SigC::Slot0<R>(f) {}
   };
 
+  //sigc::ptrfun() --> SigC::slot (for function taking no arguments)
+  template <class R>
+    slot<R> 
+    ptr_fun(R (*func)())
+    { return func; }
+
   //sigc::memfun() --> SigC::slot (for class member function taking no arguments)
   template <class R,class O1,class O2>
     slot<R>
