@@ -37,7 +37,7 @@ COPYING distributed with the source files.
 #include "socket_list.h"
 #include "dialogs.h"
 #include "socket_list_icons.h"
-#include "utf8_utils.h"
+#include "utils/utf8_utils.h"
 
 #ifdef HAVE_STRINGSTREAM
 #include <sstream>
@@ -154,7 +154,7 @@ SocketListDialog::SocketListDialog(std::pair<SharedPtr<FilenamesList>,
   GtkTreeViewColumn* column_p =
     gtk_tree_view_column_new_with_attributes(gettext("Queued print jobs"), renderer_p,
 					     "text", ModelColumns::fax_label,
-					     0);
+					     static_cast<void*>(0));
   gtk_tree_view_append_column(tree_view_p, column_p);
 
   // single line selection
@@ -327,7 +327,7 @@ void SocketListDialog::set_socket_list_rows(std::pair<SharedPtr<FilenamesList>,
   }
 }
 
-std::pair<const char*, char* const*> SocketListDialog::get_view_file_parms(const string& file_name) {
+std::pair<const char*, char* const*> SocketListDialog::get_view_file_parms(const std::string& file_name) {
 
   std::vector<std::string> view_parms;
   std::string view_cmd;
