@@ -23,7 +23,7 @@ COPYING distributed with the source files.
 #include <pthread.h>
 
 #include "socket_server.h"
-#include "shared_handle.h"
+#include "utils/shared_handle.h"
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -84,7 +84,7 @@ void SocketServer::start(const std::string& port_, bool other_sock_client_addres
 
     // this is to keep gcc-2.95.3 and libstdc++2 happy as assigning the result
     // of Thread::Thread::start() to socket_thread_h incorrectly reports an error
-    auto_ptr<Thread::Thread> temp_h =
+    std::auto_ptr<Thread::Thread> temp_h =
       Thread::Thread::start(sigc::mem_fun(*this, &SocketServer::socket_thread), true);
     socket_thread_h = temp_h;
     if (!socket_thread_h.get()) {
