@@ -20,11 +20,11 @@ COPYING distributed with the source files.
 
 #include <sigc++/sigc++.h>
 
-#include "sem_sync.h"
-#include "shared_ptr.h"
-#include "thread.h"
-#include "mutex.h"
-#include "notifier.h"
+#include "utils/sem_sync.h"
+#include "utils/shared_ptr.h"
+#include "utils/thread.h"
+#include "utils/mutex.h"
+#include "utils/notifier.h"
 
 typedef std::list<std::pair<std::string, unsigned int> > FilenamesList;
 
