@@ -20,7 +20,13 @@ COPYING distributed with the source files.
 
 #include "tray_icon.h"
 #include "efax_controller.h"
-#include "gobj_handle.h"
+#include "utils/gobj_handle.h"
+
+#ifdef HAVE_STRINGSTREAM
+#include <sstream>
+#else
+#include <strstream>
+#endif
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -54,7 +60,7 @@ gboolean TrayItemCB::button_press_cb(GtkWidget*,
   TrayItem* instance_p = static_cast<TrayItem*>(data);
   
   if (event_p->button == 1 || event_p->button == 2) instance_p->left_button_pressed();
-  if (event_p->button == 3) {
+  else if (event_p->button == 3) {
 
     if (instance_p->get_state() == EfaxController::inactive) {
       gtk_widget_set_sensitive(GTK_WIDGET(instance_p->stop_item_p), false);
@@ -251,8 +257,26 @@ void TrayItem::tray_icon_destroy(void) {
 
 void TrayItem::set_tooltip_slot(const char* text) {
 
-  std::string tooltip_text(gettext("efax-gtk: "));
-  tooltip_text += text;
-  
-  gtk_tooltips_set_tip(tooltips_p, event_box_p, tooltip_text.c_str(), "");
+#ifdef HAVE_STRINGSTREAM
+  std::ostringstream strm;
+  strm << gettext("efax-gtk: ") << text;
+  if (get_state() == EfaxController::receive_standby) {
+    strm << '\n' << gettext("New faxes:") << ' ' << get_new_fax_count();
+  }
+
+  gtk_tooltips_set_tip(tooltips_p, event_box_p, strm.str().c_str(), "");
+
+#else
+  std::ostrstream strm;
+  strm << gettext("efax-gtk: ") << text;
+  if (get_state() == EfaxController::receive_standby) {
+    strm << '\n' << gettext("New faxes:") << ' ' << get_new_fax_count();
+  }
+
+  strm << std::ends;
+  const char* tip = strm.str();
+  gtk_tooltips_set_tip(tooltips_p, event_box_p, tip, "");
+  delete[] tip;
+
+#endif
 }
